package javawrapper;

import java.util.Vector;

/**
 * This class holds a set of grid cell coordinates.
 * <p>Copyright: Copyright (c) Charles D. Canham 2003</p>
 * <p>Company: Institute of Ecosystem Studies</p>
 * @author Lora E. Murphy
 * @version 1.0
 *
 * <br>Edit history:
 * <br>------------------
 * <br>April 28, 2004: Submitted in beta version (LEM)
 * <br>February 4, 2008: Updated to conform to Java 6 compliance (LEM)
 */
public class Cell {

  protected float
      /**Length of cell in X direction.*/
      m_fXCellLength,
      /**Length of cell in Y direction.*/
      m_fYCellLength;
  protected int m_iX, /**<Cell X coordinate*/
      m_iY; /**<Cell Y coordinate*/

  /**
   * Constructor
   * @param iX X cell coordinate
   * @param iY Y cell coordinate
   * @param oPlot Plot object
   * @throws ModelException if the cell coordinates are invalid
   */
  public Cell(int iX, int iY, Plot oPlot) throws ModelException {
    this(iX, iY, (float) 8, (float) 8, oPlot);
  }

  /**
   * Constructor
   * @param iX X cell coordinate
   * @param iY Y cell coordinate
   * @param fXCellLength Cell X length, in m
   * @param fYCellLength Cell Y length, in m
   * @param oPlot Plot object
   * @throws ModelException if the cell coordinates are invalid
   */
  public Cell(int iX, int iY, float fXCellLength, float fYCellLength,
              Plot oPlot) throws ModelException {
    m_fXCellLength = fXCellLength;
    m_fYCellLength = fYCellLength;

    //Validate the grid cell coordinates
    if (iX < 0 || (iX * m_fXCellLength) > oPlot.GetPlotXLength() ||
        iY < 0 || (iY * m_fYCellLength) > oPlot.GetPlotYLength()) {
      ModelException oErr = new ModelException(ErrorGUI.BAD_DATA, "JAVA",
          "A bad set of grid cell coordinates was passed - X = " +
          String.valueOf(iX) +
          ", Y = " + String.valueOf(iY) + ".");
      throw (oErr);
    }

    m_iX = iX;
    m_iY = iY;

  }

  /**
   * Gets the X cell coordinate
   * @return X cell coordinate.
   */
  public int GetX() {
    return m_iX;
  }

  /**
   * Gets the Y cell coordinate.
   * @return Y cell coordinate.
   */
  public int GetY() {
    return m_iY;
  }

  /**
   * Transforms an old grid cell number into its equivalent new grid cells.  A
       * strict one-to-one areal overlap cannot be guaranteed unless the old and new
   * grid cells are the same size.  The list of new cells covers completely all
   * the area of the old cell, with overlap as needed to ensure the original
   * area is completely covered.
   * @param iOldGridCell The old grid cell number.
       * @param fOldXGridLength The length of X grids, in the old coordinate system.
       * @param fOldYGridLength The length of Y grids, in the old coordinate system.
   * @param iNumOldXGrids Number of grid cells in the X direction, in the old
   * coordinate system.
   * @param iNumOldYGrids Number of grid cells in the Y direction, in the old
   * coordinate system.
   * @param oPlot Plot object.
   * @return An array of Cell objects which covers the original grid cell's
   * area, or null if no equivalent list could be created.
   * @throws ModelException Passing through other underlying exceptions.
   */
  public static Cell[] TransformOldGridCell(int iOldGridCell,
                                            float fOldXGridLength,
                                            float fOldYGridLength,
                                            int iNumOldXGrids,
                                            int iNumOldYGrids, Plot oPlot) throws
      ModelException {
    Vector<Cell> oCells = new Vector<Cell>(0);
    Cell[] oReturnList;
    Cell oCell;
    float fX, fY; //coordinates of cell corner
    int i, j, //loop counter
        iOldYGrid, iOldXGrid, //the grid cell numbers in the old coordinate system
        iNewXGrid, iNewYGrid, //grid cell numbers in the new coordinate system
        iAddXGrids, iAddYGrids, //number of grid cells needed in each direction
        //to cover an old grid cell
        iNumNewXGrids = new Float( (fOldYGridLength * iNumOldYGrids) / 8).
        intValue(),
        iNumNewYGrids = new Float( (fOldXGridLength * iNumOldXGrids) / 8).
        intValue();

    //Coordinates are reversed
    if (fOldXGridLength > 8) {
      iAddYGrids = new Float(java.lang.Math.floor( (double)
                                                  fOldXGridLength / 8)).
          intValue();
    }
    else {
      iAddYGrids = 0;
    }
    if (fOldYGridLength > 8) {
      iAddXGrids = new Float(java.lang.Math.floor( (double)
                                                  fOldYGridLength / 8)).
          intValue();
    }
    else {
      iAddXGrids = 0;
    }

    //Grids are calculated in the old coordinate system as
    //grid = xgrid*numYgrids + ygrid
    iOldXGrid = iOldGridCell / iNumOldYGrids;
    iOldYGrid = iOldGridCell - (iOldXGrid * iNumOldYGrids);

    //Find the coordinates of the origin of this cell in the new coordinate system
    fX = iOldYGrid * fOldYGridLength;
    fY = iOldXGrid * fOldXGridLength;

    //Transform into new cell numbers
    iNewXGrid = new Float(fX / 8).intValue();
    iNewYGrid = new Float(fY / 8).intValue();

    //Add this cell
    oCells.add(new Cell(iNewXGrid, iNewYGrid, oPlot));

    //If the length of cells in either direction is greater than
    //the new cell length, include those cells too
    if (fOldXGridLength > 8 || fOldYGridLength > 8) {
      for (i = 0; i <= iAddXGrids; i++) {
        for (j = 0; j <= iAddYGrids; j++) {
          if (i != 0 || j != 0) { //so we don't add our first one again
            if (iNewXGrid + i < iNumNewXGrids &&
                iNewYGrid + j < iNumNewYGrids) {
              oCells.add(new Cell(iNewXGrid + i, iNewYGrid + j, oPlot));
            }
          }
        }
      }
    }

    if (oCells.size() == 0) {
      return null;
    }

    oReturnList = new Cell[oCells.size()];
    for (i = 0; i < oReturnList.length; i++) {
      oCell = (Cell) oCells.elementAt(i);
      oReturnList[i] = oCell;
    }

    return oReturnList;

  }
}
