package javawrapper;

/**
 * Manages substrate data and behaviors.
 * <p>Copyright: Copyright (c) 2003 Charles D. Canham</p>
 * <p>Company: Institute of Ecosystem Studies</p>
 * @author Lora E. Murphy
 *
 * <br>Edit history:
 * <br>------------------
 * <br>April 28, 2004: Submitted in beta version (LEM)
 * <br>July 21, 2004:  Added calcs grids (LEM)
 * <br>April 1, 2005:  Changed some substrate names per CDC (LEM)
 * <br>April 22, 2005:  Added new snag uproot parameter (LEM)
 * <br>June 3, 2005:  Added combined substrate (LEM)
 * <br>November 17, 2005:  New decay equation for decayed logs (LEM)
 * <br>November 16, 2007: Added detailed substrate behavior (LEM)
 */

public class SubstrateBehaviors
    extends BehaviorTypeBase {

	/**Substrate - Proportion of dead that fall for each species.*/
  protected ModelVector mp_fProportionOfDeadThatFall = new ModelVector(
      "Proportion of Dead that Fall", "su_propOfDeadFall", "su_podfVal", 0,
      ModelVector.FLOAT);

  /**Substrate - Proportion of fallen that uproot for each species.*/
  protected ModelVector mp_fProportionOfFallenThatUproot = new ModelVector(
      "Proportion of Fallen that Uproot", "su_propOfFallUproot", "su_pofuVal",
      0, ModelVector.FLOAT);

  /**Substrate - Proportion of snags that uproot for each species. LEM 04/22/05*/
  protected ModelVector mp_fProportionOfSnagsThatUproot = new ModelVector(
      "Proportion of Snags that Uproot", "su_propOfSnagsUproot", "su_posuVal",
      0, ModelVector.FLOAT);

  /**Substrate - Substrate - Scarified soil decay alpha*/
  protected ModelFloat m_fScarSoilA = new ModelFloat(0, 
  		"Scarified Soil Annual Decay Alpha", "su_scarSoilDecayAlpha");

  /**Substrate - Scarified soil decay beta*/
  protected ModelFloat m_fScarSoilB = new ModelFloat(0,
      "Scarified Soil Annual Decay Beta",
      "su_scarSoilDecayBeta");

  /**Substrate - Tip-Up Mounds decay alpha*/
  protected ModelFloat m_fTipUpA = new ModelFloat(0,
      "Tip-Up Mounds Annual Decay Alpha",
      "su_tipupDecayAlpha");

  /**Substrate - Tip-Up Mounds decay beta*/
  protected ModelFloat m_fTipUpB = new ModelFloat(0,
      "Tip-Up Mounds Annual Decay Beta",
      "su_tipupDecayBeta");

  /**Substrate - Fresh log decay alpha*/
  protected ModelFloat m_fFreshlogA = new ModelFloat(0,
      "Fresh Log Annual Decay Alpha", "su_freshLogDecayAlpha");

  /**Substrate - Fresh log decay beta*/
  protected ModelFloat m_fFreshlogB = new ModelFloat(0,
      "Fresh Log Annual Decay Beta", "su_freshLogDecayBeta");

  /**Substrate - Decayed log decay alpha*/
  protected ModelFloat m_fDecayedlogA = new ModelFloat(0,
      "Decayed Log Annual Decay Alpha", "su_decayedLogDecayAlpha");

  /**Substrate - Decayed log decay beta*/
  protected ModelFloat m_fDecayedlogB = new ModelFloat(0,
      "Decayed Log Annual Decay Beta", "su_decayedLogDecayBeta");

  /**Substrate - Scarified soil initial condition proportion*/
  protected ModelFloat m_fInitCondScarSoil = new ModelFloat(0,
      "Initial Conditions Proportion of Scarified Soil", "su_initialScarSoil");

  /**Substrate - Scarified soil partial cut proportion*/
  protected ModelFloat m_fPartialCutScarSoil = new ModelFloat(0,
      "Partial Cut Proportion of Scarified Soil",
      "su_partialCutScarSoil");

  /**Substrate - Scarified soil gap cut proportion*/
  protected ModelFloat m_fGapCutScarSoil = new ModelFloat(0,
      "Gap Cut Proportion of Scarified Soil",
      "su_gapCutScarSoil");

  /**Substrate - Scarified soil clear cut proportion*/
  protected ModelFloat m_fClearCutScarSoil = new ModelFloat(0,
      "Clear Cut Proportion of Scarified Soil",
      "su_clearCutScarSoil");

  /**Substrate - Tip-Up Mounds initial condition proportion*/
  protected ModelFloat m_fInitCondTipup = new ModelFloat(0,
      "Initial Conditions Proportion of Tip-Up Mounds", "su_initialTipup");

  /**Substrate - Tip-Up Mounds partial cut proportion*/
  protected ModelFloat m_fPartialCutTipup = new ModelFloat(0,
      "Partial Cut Proportion of Tip-Up Mounds",
      "su_partialCutTipup");

  /**Substrate - Tip-Up Mounds gap cut proportion*/
  protected ModelFloat m_fGapCutTipup = new ModelFloat(0,
      "Gap Cut Proportion of Tip-Up Mounds",
      "su_gapCutTipup");

  /**Substrate - Tip-Up Mounds clear cut proportion*/
  protected ModelFloat m_fClearCutTipup = new ModelFloat(0,
      "Clear Cut Proportion of Tip-Up Mounds",
      "su_clearCutTipup");

  /**Substrate - Fresh log initial condition proportion*/
  protected ModelFloat m_fInitCondFreshLog = new ModelFloat(0,
      "Initial Conditions Proportion of Fresh Logs", "su_initialFreshLog");

  /**Substrate - Fresh log partial cut proportion*/
  protected ModelFloat m_fPartialCutFreshLog = new ModelFloat(0,
      "Partial Cut Proportion of Fresh Logs", "su_partialCutFreshLog");

  /**Substrate - Fresh log gap cut proportion*/
  protected ModelFloat m_fGapCutFreshLog = new ModelFloat(0,
      "Gap Cut Proportion of Fresh Logs",
      "su_gapCutFreshLog");

  /**Substrate - Fresh log clear cut proportion*/
  protected ModelFloat m_fClearCutFreshLog = new ModelFloat(0,
      "Clear Cut Proportion of Fresh Logs",
      "su_clearCutFreshLog");

  /**Substrate - Decayed log initial condition proportion*/
  protected ModelFloat m_fInitCondDecLog = new ModelFloat(0,
      "Initial Conditions Proportion of Decayed Logs", "su_initialDecayedLog");

  /**Substrate - Decayed log partial cut proportion*/
  protected ModelFloat m_fPartialCutDecLog = new ModelFloat(0,
      "Partial Cut Proportion of Decayed Logs", "su_partialCutDecayedLog");

  /**Substrate - Decayed log gap cut proportion*/
  protected ModelFloat m_fGapCutDecLog = new ModelFloat(0,
      "Gap Cut Proportion of Decayed Logs",
      "su_gapCutDecayedLog");

  /**Substrate - Decayed log clear cut proportion*/
  protected ModelFloat m_fClearCutDecLog = new ModelFloat(0,
      "Clear Cut Proportion of Decayed Logs",
      "su_clearCutDecayedLog");

  /**Substrate - Root soil disturbance factor*/
  protected ModelFloat m_fRootTipupFactor = new ModelFloat(0,
      "Uprooted Tree Radius Increase Factor for Root Rip-Out",
      "su_rootTipupFactor");

  /**Substrate - Proportion of litter/moss that is moss*/
  protected ModelFloat m_fMossProportion = new ModelFloat(0,
      "Proportion of Forest Floor Litter/Moss Pool that is Moss",
      "su_mossProportion");

  /**Substrate - Whether or not tree fall is directional - LEM 06/03/05*/
  protected ModelEnum m_iDirectionalTreeFall =
      new ModelEnum(new int[] {0, 1},
                    new String[] {"false", "true"},
                    "Use Directional Tree Fall",
                    "su_directionalTreeFall");

  /**Substrate - Maximum number of years a substrate event hangs around*/
  protected ModelInt m_iMaxDecayTime = new ModelInt(10,
                                     "Maximum Number of Years that Decay Occurs",
                                     "su_maxNumberDecayYears");
  
  /** Detailed substrate - Boundary between two log diameter classes, in cm */
  protected ModelFloat m_fLogSizeClassBoundary = new ModelFloat(0, 
  		"Boundary Between Log Diam Classes (cm)", "su_logSizeClassBoundary");
  
  /** Detailed substrate - species group ID for each species */
	protected ModelVector mp_iSpeciesGroup = new ModelVector(
			"Species Group", "su_logSpGroupAssignment", "su_lsgaVal", 0,
			ModelVector.MODEL_ENUM);
	
	/**Detailed substrate - proportion of live trees that enter decay class 1*/
	protected ModelFloat m_fPropFallenTreesDecayClass1 = new ModelFloat(0,
      "Prop. Live Trees Entering Decay Class 1 (0-1)", 
      "su_propFallenTreesLogDecayClass1");
	
	/**Detailed substrate - proportion of live trees that enter decay class 2*/
	protected ModelFloat m_fPropFallenTreesDecayClass2 = new ModelFloat(0,
      "Prop. Live Trees Entering Decay Class 2 (0-1)", 
      "su_propFallenTreesLogDecayClass2");

	/**Detailed substrate - proportion of live trees that enter decay class 3*/
	protected ModelFloat m_fPropFallenTreesDecayClass3 = new ModelFloat(0,
      "Prop. Live Trees Entering Decay Class 3 (0-1)", 
      "su_propFallenTreesLogDecayClass3");
	
	/**Detailed substrate - proportion of live trees that enter decay class 4*/
	protected ModelFloat m_fPropFallenTreesDecayClass4 = new ModelFloat(0,
      "Prop. Live Trees Entering Decay Class 4 (0-1)", 
      "su_propFallenTreesLogDecayClass4");
	
	/**Detailed substrate - proportion of live trees that enter decay class 5*/
	protected ModelFloat m_fPropFallenTreesDecayClass5 = new ModelFloat(0,
      "Prop. Live Trees Entering Decay Class 5 (0-1)", 
      "su_propFallenTreesLogDecayClass5");
	
	/**Detailed substrate - proportion of snags that enter decay class 1*/
	protected ModelFloat m_fPropFallenSnagsDecayClass1 = new ModelFloat(0,
      "Prop. Snags Entering Decay Class 1 (0-1)", 
      "su_propFallenSnagsLogDecayClass1");
	
	/**Detailed substrate - proportion of snags that enter decay class 2*/
	protected ModelFloat m_fPropFallenSnagsDecayClass2 = new ModelFloat(0,
      "Prop. Snags Entering Decay Class 2 (0-1)", 
      "su_propFallenSnagsLogDecayClass2");
	
	/**Detailed substrate - proportion of snags that enter decay class 3*/
	protected ModelFloat m_fPropFallenSnagsDecayClass3 = new ModelFloat(0,
      "Prop. Snags Entering Decay Class 3 (0-1)", 
      "su_propFallenSnagsLogDecayClass3");
	
	/**Detailed substrate - proportion of snags that enter decay class 4*/
	protected ModelFloat m_fPropFallenSnagsDecayClass4 = new ModelFloat(0,
      "Prop. Snags Entering Decay Class 4 (0-1)", 
      "su_propFallenSnagsLogDecayClass4");
	
	/**Detailed substrate - proportion of snags that enter decay class 5*/
	protected ModelFloat m_fPropFallenSnagsDecayClass5 = new ModelFloat(0,
      "Prop. Snags Entering Decay Class 5 (0-1)", 
      "su_propFallenSnagsLogDecayClass5");
	
	/**Detailed substrate - species group 1 small class 1 log decay alpha*/
	protected ModelFloat m_fSpGroup1SmallClass1Alpha = new ModelFloat(0,
      "Species Group 1 Small Class 1 Log Decay Alpha", 
      "su_logSpGroup1SmallDecayClass1DecayAlpha");
	
	/**Detailed substrate - species group 1 small class 2 log decay alpha*/
	protected ModelFloat m_fSpGroup1SmallClass2Alpha = new ModelFloat(0,
      "Species Group 1 Small Class 2 Log Decay Alpha", 
      "su_logSpGroup1SmallDecayClass2DecayAlpha");
	
	/**Detailed substrate - species group 1 small class 3 log decay alpha*/
	protected ModelFloat m_fSpGroup1SmallClass3Alpha = new ModelFloat(0,
      "Species Group 1 Small Class 3 Log Decay Alpha", 
      "su_logSpGroup1SmallDecayClass3DecayAlpha");
	
	/**Detailed substrate - species group 1 small class 4 log decay alpha*/
	protected ModelFloat m_fSpGroup1SmallClass4Alpha = new ModelFloat(0,
      "Species Group 1 Small Class 4 Log Decay Alpha", 
      "su_logSpGroup1SmallDecayClass4DecayAlpha");
	
	/**Detailed substrate - species group 1 small class 5 log decay alpha*/
	protected ModelFloat m_fSpGroup1SmallClass5Alpha = new ModelFloat(0,
      "Species Group 1 Small Class 5 Log Decay Alpha", 
      "su_logSpGroup1SmallDecayClass5DecayAlpha");
	
	/**Detailed substrate - species group 2 small class 1 log decay alpha*/
	protected ModelFloat m_fSpGroup2SmallClass1Alpha = new ModelFloat(0,
      "Species Group 2 Small Class 1 Log Decay Alpha", 
      "su_logSpGroup2SmallDecayClass1DecayAlpha");
	
	/**Detailed substrate - species group 2 small class 2 log decay alpha*/
	protected ModelFloat m_fSpGroup2SmallClass2Alpha = new ModelFloat(0,
      "Species Group 2 Small Class 2 Log Decay Alpha", 
      "su_logSpGroup2SmallDecayClass2DecayAlpha");
	
	/**Detailed substrate - species group 2 small class 3 log decay alpha*/
	protected ModelFloat m_fSpGroup2SmallClass3Alpha = new ModelFloat(0,
      "Species Group 2 Small Class 3 Log Decay Alpha", 
      "su_logSpGroup2SmallDecayClass3DecayAlpha");
	
	/**Detailed substrate - species group 2 small class 4 log decay alpha*/
	protected ModelFloat m_fSpGroup2SmallClass4Alpha = new ModelFloat(0,
      "Species Group 2 Small Class 4 Log Decay Alpha", 
      "su_logSpGroup2SmallDecayClass4DecayAlpha");
	
	/**Detailed substrate - species group 2 small class 5 log decay alpha*/
	protected ModelFloat m_fSpGroup2SmallClass5Alpha = new ModelFloat(0,
      "Species Group 2 Small Class 5 Log Decay Alpha", 
      "su_logSpGroup2SmallDecayClass5DecayAlpha");
	
	/**Detailed substrate - species group 3 small class 1 log decay alpha*/
	protected ModelFloat m_fSpGroup3SmallClass1Alpha = new ModelFloat(0,
      "Species Group 3 Small Class 1 Log Decay Alpha", 
      "su_logSpGroup3SmallDecayClass1DecayAlpha");
	
	/**Detailed substrate - species group 3 small class 2 log decay alpha*/
	protected ModelFloat m_fSpGroup3SmallClass2Alpha = new ModelFloat(0,
      "Species Group 3 Small Class 2 Log Decay Alpha", 
      "su_logSpGroup3SmallDecayClass2DecayAlpha");
	
	/**Detailed substrate - species group 3 small class 3 log decay alpha*/
	protected ModelFloat m_fSpGroup3SmallClass3Alpha = new ModelFloat(0,
      "Species Group 3 Small Class 3 Log Decay Alpha", 
      "su_logSpGroup3SmallDecayClass3DecayAlpha");
	
	/**Detailed substrate - species group 3 small class 4 log decay alpha*/
	protected ModelFloat m_fSpGroup3SmallClass4Alpha = new ModelFloat(0,
      "Species Group 3 Small Class 4 Log Decay Alpha", 
      "su_logSpGroup3SmallDecayClass4DecayAlpha");
	
	/**Detailed substrate - species group 3 small class 5 log decay alpha*/
	protected ModelFloat m_fSpGroup3SmallClass5Alpha = new ModelFloat(0,
      "Species Group 3 Small Class 5 Log Decay Alpha", 
      "su_logSpGroup3SmallDecayClass5DecayAlpha");
	
	/**Detailed substrate - species group 1 large class 1 log decay alpha*/
	protected ModelFloat m_fSpGroup1LargeClass1Alpha = new ModelFloat(0,
      "Species Group 1 Large Class 1 Log Decay Alpha", 
      "su_logSpGroup1LargeDecayClass1DecayAlpha");
	
	/**Detailed substrate - species group 1 large class 2 log decay alpha*/
	protected ModelFloat m_fSpGroup1LargeClass2Alpha = new ModelFloat(0,
      "Species Group 1 Large Class 2 Log Decay Alpha", 
      "su_logSpGroup1LargeDecayClass2DecayAlpha");
	
	/**Detailed substrate - species group 1 large class 3 log decay alpha*/
	protected ModelFloat m_fSpGroup1LargeClass3Alpha = new ModelFloat(0,
      "Species Group 1 Large Class 3 Log Decay Alpha", 
      "su_logSpGroup1LargeDecayClass3DecayAlpha");
	
	/**Detailed substrate - species group 1 large class 4 log decay alpha*/
	protected ModelFloat m_fSpGroup1LargeClass4Alpha = new ModelFloat(0,
      "Species Group 1 Large Class 4 Log Decay Alpha", 
      "su_logSpGroup1LargeDecayClass4DecayAlpha");
	
	/**Detailed substrate - species group 1 large class 5 log decay alpha*/
	protected ModelFloat m_fSpGroup1LargeClass5Alpha = new ModelFloat(0,
      "Species Group 1 Large Class 5 Log Decay Alpha", 
      "su_logSpGroup1LargeDecayClass5DecayAlpha");
	
	/**Detailed substrate - species group 2 large class 1 log decay alpha*/
	protected ModelFloat m_fSpGroup2LargeClass1Alpha = new ModelFloat(0,
      "Species Group 2 Large Class 1 Log Decay Alpha", 
      "su_logSpGroup2LargeDecayClass1DecayAlpha");
	
	/**Detailed substrate - species group 2 large class 2 log decay alpha*/
	protected ModelFloat m_fSpGroup2LargeClass2Alpha = new ModelFloat(0,
      "Species Group 2 Large Class 2 Log Decay Alpha", 
      "su_logSpGroup2LargeDecayClass2DecayAlpha");
	
	/**Detailed substrate - species group 2 large class 3 log decay alpha*/
	protected ModelFloat m_fSpGroup2LargeClass3Alpha = new ModelFloat(0,
      "Species Group 2 Large Class 3 Log Decay Alpha", 
      "su_logSpGroup2LargeDecayClass3DecayAlpha");
	
	/**Detailed substrate - species group 2 large class 4 log decay alpha*/
	protected ModelFloat m_fSpGroup2LargeClass4Alpha = new ModelFloat(0,
      "Species Group 2 Large Class 4 Log Decay Alpha", 
      "su_logSpGroup2LargeDecayClass4DecayAlpha");
	
	/**Detailed substrate - species group 2 large class 5 log decay alpha*/
	protected ModelFloat m_fSpGroup2LargeClass5Alpha = new ModelFloat(0,
      "Species Group 2 Large Class 5 Log Decay Alpha", 
      "su_logSpGroup2LargeDecayClass5DecayAlpha");
	
	/**Detailed substrate - species group 3 large class 1 log decay alpha*/
	protected ModelFloat m_fSpGroup3LargeClass1Alpha = new ModelFloat(0,
      "Species Group 3 Large Class 1 Log Decay Alpha",
      "su_logSpGroup3LargeDecayClass1DecayAlpha");
	
	/**Detailed substrate - species group 3 large class 2 log decay alpha*/
	protected ModelFloat m_fSpGroup3LargeClass2Alpha = new ModelFloat(0,
      "Species Group 3 Large Class 2 Log Decay Alpha", 
      "su_logSpGroup3LargeDecayClass2DecayAlpha");
	
	/**Detailed substrate - species group 3 large class 3 log decay alpha*/
	protected ModelFloat m_fSpGroup3LargeClass3Alpha = new ModelFloat(0,
      "Species Group 3 Large Class 3 Log Decay Alpha", 
      "su_logSpGroup3LargeDecayClass3DecayAlpha");

	/**Detailed substrate - species group 3 large class 4 log decay alpha*/
	protected ModelFloat m_fSpGroup3LargeClass4Alpha = new ModelFloat(0,
      "Species Group 3 Large Class 4 Log Decay Alpha", 
      "su_logSpGroup3LargeDecayClass4DecayAlpha");
	
	/**Detailed substrate - species group 3 large class 5 log decay alpha*/
	protected ModelFloat m_fSpGroup3LargeClass5Alpha = new ModelFloat(0,
      "Species Group 3 Large Class 5 Log Decay Alpha", 
      "su_logSpGroup3LargeDecayClass5DecayAlpha");
	
	/**Detailed substrate - species group 1 small class 1 log decay beta*/
	protected ModelFloat m_fSpGroup1SmallClass1Beta = new ModelFloat(0,
      "Species Group 1 Small Class 1 Log Decay Beta", 
      "su_logSpGroup1SmallDecayClass1DecayBeta");
	
	/**Detailed substrate - species group 1 small class 2 log decay beta*/
	protected ModelFloat m_fSpGroup1SmallClass2Beta = new ModelFloat(0,
      "Species Group 1 Small Class 2 Log Decay Beta", 
      "su_logSpGroup1SmallDecayClass2DecayBeta");
	
	/**Detailed substrate - species group 1 small class 3 log decay beta*/
	protected ModelFloat m_fSpGroup1SmallClass3Beta = new ModelFloat(0,
      "Species Group 1 Small Class 3 Log Decay Beta", 
      "su_logSpGroup1SmallDecayClass3DecayBeta");
	
	/**Detailed substrate - species group 1 small class 4 log decay beta*/
	protected ModelFloat m_fSpGroup1SmallClass4Beta = new ModelFloat(0,
      "Species Group 1 Small Class 4 Log Decay Beta", 
      "su_logSpGroup1SmallDecayClass4DecayBeta");

	/**Detailed substrate - species group 1 small class 5 log decay beta*/
	protected ModelFloat m_fSpGroup1SmallClass5Beta = new ModelFloat(0,
      "Species Group 1 Small Class 5 Log Decay Beta", 
      "su_logSpGroup1SmallDecayClass5DecayBeta");
	
	/**Detailed substrate - species group 2 small class 1 log decay beta*/
	protected ModelFloat m_fSpGroup2SmallClass1Beta = new ModelFloat(0,
      "Species Group 2 Small Class 1 Log Decay Beta", 
      "su_logSpGroup2SmallDecayClass1DecayBeta");
	
	/**Detailed substrate - species group 2 small class 2 log decay beta*/
	protected ModelFloat m_fSpGroup2SmallClass2Beta = new ModelFloat(0,
      "Species Group 2 Small Class 2 Log Decay Beta", 
      "su_logSpGroup2SmallDecayClass2DecayBeta");
	
	/**Detailed substrate - species group 2 small class 3 log decay beta*/
	protected ModelFloat m_fSpGroup2SmallClass3Beta = new ModelFloat(0,
      "Species Group 2 Small Class 3 Log Decay Beta", 
      "su_logSpGroup2SmallDecayClass3DecayBeta");
	
	/**Detailed substrate - species group 2 small class 4 log decay beta*/
	protected ModelFloat m_fSpGroup2SmallClass4Beta = new ModelFloat(0,
      "Species Group 2 Small Class 4 Log Decay Beta", 
      "su_logSpGroup2SmallDecayClass4DecayBeta");
	
	/**Detailed substrate - species group 2 small class 5 log decay beta*/
	protected ModelFloat m_fSpGroup2SmallClass5Beta = new ModelFloat(0,
      "Species Group 2 Small Class 5 Log Decay Beta", 
      "su_logSpGroup2SmallDecayClass5DecayBeta");
	
	/**Detailed substrate - species group 3 small class 1 log decay beta*/
	protected ModelFloat m_fSpGroup3SmallClass1Beta = new ModelFloat(0,
      "Species Group 3 Small Class 1 Log Decay Beta", 
      "su_logSpGroup3SmallDecayClass1DecayBeta");
	
	/**Detailed substrate - species group 3 small class 2 log decay beta*/
	protected ModelFloat m_fSpGroup3SmallClass2Beta = new ModelFloat(0,
      "Species Group 3 Small Class 2 Log Decay Beta", 
      "su_logSpGroup3SmallDecayClass2DecayBeta");
	
	/**Detailed substrate - species group 3 small class 3 log decay beta*/
	protected ModelFloat m_fSpGroup3SmallClass3Beta = new ModelFloat(0,
      "Species Group 3 Small Class 3 Log Decay Beta", 
      "su_logSpGroup3SmallDecayClass3DecayBeta");
	
	/**Detailed substrate - species group 3 small class 4 log decay beta*/
	protected ModelFloat m_fSpGroup3SmallClass4Beta = new ModelFloat(0,
      "Species Group 3 Small Class 4 Log Decay Beta", 
      "su_logSpGroup3SmallDecayClass4DecayBeta");
	
	/**Detailed substrate - species group 3 small class 5 log decay beta*/
	protected ModelFloat m_fSpGroup3SmallClass5Beta = new ModelFloat(0,
      "Species Group 3 Small Class 5 Log Decay Beta", 
      "su_logSpGroup3SmallDecayClass5DecayBeta");
	
	/**Detailed substrate - species group 1 large class 1 log decay beta*/
	protected ModelFloat m_fSpGroup1LargeClass1Beta = new ModelFloat(0,
      "Species Group 1 Large Class 1 Log Decay Beta", 
      "su_logSpGroup1LargeDecayClass1DecayBeta");
	
	/**Detailed substrate - species group 1 large class 2 log decay beta*/
	protected ModelFloat m_fSpGroup1LargeClass2Beta = new ModelFloat(0,
      "Species Group 1 Large Class 2 Log Decay Beta", 
      "su_logSpGroup1LargeDecayClass2DecayBeta");
	
	/**Detailed substrate - species group 1 large class 3 log decay beta*/
	protected ModelFloat m_fSpGroup1LargeClass3Beta = new ModelFloat(0,
      "Species Group 1 Large Class 3 Log Decay Beta", 
      "su_logSpGroup1LargeDecayClass3DecayBeta");
	
	/**Detailed substrate - species group 1 large class 4 log decay beta*/
	protected ModelFloat m_fSpGroup1LargeClass4Beta = new ModelFloat(0,
      "Species Group 1 Large Class 4 Log Decay Beta", 
      "su_logSpGroup1LargeDecayClass4DecayBeta");
	
	/**Detailed substrate - species group 1 large class 5 log decay beta*/
	protected ModelFloat m_fSpGroup1LargeClass5Beta = new ModelFloat(0,
      "Species Group 1 Large Class 5 Log Decay Beta", 
      "su_logSpGroup1LargeDecayClass5DecayBeta");
	
	/**Detailed substrate - species group 2 large class 1 log decay beta*/
	protected ModelFloat m_fSpGroup2LargeClass1Beta = new ModelFloat(0,
      "Species Group 2 Large Class 1 Log Decay Beta", 
      "su_logSpGroup2LargeDecayClass1DecayBeta");
	
	/**Detailed substrate - species group 2 large class 2 log decay beta*/
	protected ModelFloat m_fSpGroup2LargeClass2Beta = new ModelFloat(0,
      "Species Group 2 Large Class 2 Log Decay Beta", 
      "su_logSpGroup2LargeDecayClass2DecayBeta");
	
	/**Detailed substrate - species group 2 large class 3 log decay beta*/
	protected ModelFloat m_fSpGroup2LargeClass3Beta = new ModelFloat(0,
      "Species Group 2 Large Class 3 Log Decay Beta", 
      "su_logSpGroup2LargeDecayClass3DecayBeta");
	
	/**Detailed substrate - species group 2 large class 4 log decay beta*/
	protected ModelFloat m_fSpGroup2LargeClass4Beta = new ModelFloat(0,
      "Species Group 2 Large Class 4 Log Decay Beta", 
      "su_logSpGroup2LargeDecayClass4DecayBeta");
	
	/**Detailed substrate - species group 2 large class 5 log decay beta*/
	protected ModelFloat m_fSpGroup2LargeClass5Beta = new ModelFloat(0,
      "Species Group 2 Large Class 5 Log Decay Beta", 
      "su_logSpGroup2LargeDecayClass5DecayBeta");
	
	/**Detailed substrate - species group 3 large class 1 log decay beta*/
	protected ModelFloat m_fSpGroup3LargeClass1Beta = new ModelFloat(0,
      "Species Group 3 Large Class 1 Log Decay Beta", 
      "su_logSpGroup3LargeDecayClass1DecayBeta");
	
	/**Detailed substrate - species group 3 large class 2 log decay beta*/
	protected ModelFloat m_fSpGroup3LargeClass2Beta = new ModelFloat(0,
      "Species Group 3 Large Class 2 Log Decay Beta", 
      "su_logSpGroup3LargeDecayClass2DecayBeta");
	
	/**Detailed substrate - species group 3 large class 3 log decay beta*/
	protected ModelFloat m_fSpGroup3LargeClass3Beta = new ModelFloat(0,
      "Species Group 3 Large Class 3 Log Decay Beta", 
      "su_logSpGroup3LargeDecayClass3DecayBeta");
	
	/**Detailed substrate - species group 3 large class 4 log decay beta*/
	protected ModelFloat m_fSpGroup3LargeClass4Beta = new ModelFloat(0,
      "Species Group 3 Large Class 4 Log Decay Beta", 
      "su_logSpGroup3LargeDecayClass4DecayBeta");
	
	/**Detailed substrate - species group 3 large class 5 log decay beta*/
	protected ModelFloat m_fSpGroup3LargeClass5Beta = new ModelFloat(0,
      "Species Group 3 Large Class 5 Log Decay Beta", 
      "su_logSpGroup3LargeDecayClass5DecayBeta");
	
	/**Detailed substrate - species group 1 small class 1 log initial 
	 * conditions proportion*/
	protected ModelFloat m_fSpGroup1SmallClass1InitLog = new ModelFloat(0,
			"Species Group 1 Small Class 1 Initial Log Prop (0-1)",
			"su_initialLogSpGroup1SmallDecayClass1");
	
	/**Detailed substrate - species group 1 small class 2 log initial 
	 * conditions proportion*/
	protected ModelFloat m_fSpGroup1SmallClass2InitLog = new ModelFloat(0,
			"Species Group 1 Small Class 2 Initial Log Prop (0-1)",
			"su_initialLogSpGroup1SmallDecayClass2");
	
	/**Detailed substrate - species group 1 small class 3 log initial 
	 * conditions proportion*/
	protected ModelFloat m_fSpGroup1SmallClass3InitLog = new ModelFloat(0,
			"Species Group 1 Small Class 3 Initial Log Prop (0-1)",
			"su_initialLogSpGroup1SmallDecayClass3");
	
	/**Detailed substrate - species group 1 small class 4 log initial 
	 * conditions proportion*/
	protected ModelFloat m_fSpGroup1SmallClass4InitLog = new ModelFloat(0,
			"Species Group 1 Small Class 4 Initial Log Prop (0-1)",
			"su_initialLogSpGroup1SmallDecayClass4");
	
	/**Detailed substrate - species group 1 small class 5 log initial 
	 * conditions proportion*/
	protected ModelFloat m_fSpGroup1SmallClass5InitLog = new ModelFloat(0,
			"Species Group 1 Small Class 5 Initial Log Prop (0-1)",
			"su_initialLogSpGroup1SmallDecayClass5");
	
	/**Detailed substrate - species group 2 small class 1 log initial 
	 * conditions proportion*/
	protected ModelFloat m_fSpGroup2SmallClass1InitLog = new ModelFloat(0,
			"Species Group 2 Small Class 1 Initial Log Prop (0-1)",
			"su_initialLogSpGroup2SmallDecayClass1");
	
	/**Detailed substrate - species group 2 small class 2 log initial 
	 * conditions proportion*/
	protected ModelFloat m_fSpGroup2SmallClass2InitLog = new ModelFloat(0,
			"Species Group 2 Small Class 2 Initial Log Prop (0-1)",
			"su_initialLogSpGroup2SmallDecayClass2");
	
	/**Detailed substrate - species group 2 small class 3 log initial 
	 * conditions proportion*/
	protected ModelFloat m_fSpGroup2SmallClass3InitLog = new ModelFloat(0,
			"Species Group 2 Small Class 3 Initial Log Prop (0-1)",
			"su_initialLogSpGroup2SmallDecayClass3");
	
	/**Detailed substrate - species group 2 small class 4 log initial 
	 * conditions proportion*/
	protected ModelFloat m_fSpGroup2SmallClass4InitLog = new ModelFloat(0,
			"Species Group 2 Small Class 4 Initial Log Prop (0-1)",
			"su_initialLogSpGroup2SmallDecayClass4");
	
	/**Detailed substrate - species group 2 small class 5 log initial 
	 * conditions proportion*/
	protected ModelFloat m_fSpGroup2SmallClass5InitLog = new ModelFloat(0,
			"Species Group 2 Small Class 5 Initial Log Prop (0-1)",
			"su_initialLogSpGroup2SmallDecayClass5");
	
	/**Detailed substrate - species group 3 small class 1 log initial 
	 * conditions proportion*/
	protected ModelFloat m_fSpGroup3SmallClass1InitLog = new ModelFloat(0,
			"Species Group 3 Small Class 1 Initial Log Prop (0-1)",
			"su_initialLogSpGroup3SmallDecayClass1");
	
	/**Detailed substrate - species group 3 small class 2 log initial 
	 * conditions proportion*/
	protected ModelFloat m_fSpGroup3SmallClass2InitLog = new ModelFloat(0,
			"Species Group 3 Small Class 2 Initial Log Prop (0-1)",
			"su_initialLogSpGroup3SmallDecayClass2");
	
	/**Detailed substrate - species group 3 small class 3 log initial 
	 * conditions proportion*/
	protected ModelFloat m_fSpGroup3SmallClass3InitLog = new ModelFloat(0,
			"Species Group 3 Small Class 3 Initial Log Prop (0-1)",
			"su_initialLogSpGroup3SmallDecayClass3");
	
	/**Detailed substrate - species group 3 small class 4 log initial 
	 * conditions proportion*/
	protected ModelFloat m_fSpGroup3SmallClass4InitLog = new ModelFloat(0,
			"Species Group 3 Small Class 4 Initial Log Prop (0-1)",
			"su_initialLogSpGroup3SmallDecayClass4");
	
	/**Detailed substrate - species group 3 small class 5 log initial 
	 * conditions proportion*/
	protected ModelFloat m_fSpGroup3SmallClass5InitLog = new ModelFloat(0,
			"Species Group 3 Small Class 5 Initial Log Prop (0-1)",
			"su_initialLogSpGroup3SmallDecayClass5");
	
	/**Detailed substrate - species group 1 large class 1 log initial 
	 * conditions proportion*/
	protected ModelFloat m_fSpGroup1LargeClass1InitLog = new ModelFloat(0,
			"Species Group 1 Large Class 1 Initial Log Prop (0-1)",
			"su_initialLogSpGroup1LargeDecayClass1");
	
	/**Detailed substrate - species group 1 large class 2 log initial 
	 * conditions proportion*/
	protected ModelFloat m_fSpGroup1LargeClass2InitLog = new ModelFloat(0,
			"Species Group 1 Large Class 2 Initial Log Prop (0-1)",
			"su_initialLogSpGroup1LargeDecayClass2");
	
	/**Detailed substrate - species group 1 large class 3 log initial 
	 * conditions proportion*/
	protected ModelFloat m_fSpGroup1LargeClass3InitLog = new ModelFloat(0,
			"Species Group 1 Large Class 3 Initial Log Prop (0-1)",
			"su_initialLogSpGroup1LargeDecayClass3");
	
	/**Detailed substrate - species group 1 large class 4 log initial 
	 * conditions proportion*/
	protected ModelFloat m_fSpGroup1LargeClass4InitLog = new ModelFloat(0,
			"Species Group 1 Large Class 4 Initial Log Prop (0-1)",
			"su_initialLogSpGroup1LargeDecayClass4");
	
	/**Detailed substrate - species group 1 large class 5 log initial 
	 * conditions proportion*/
	protected ModelFloat m_fSpGroup1LargeClass5InitLog = new ModelFloat(0,
			"Species Group 1 Large Class 5 Initial Log Prop (0-1)",
			"su_initialLogSpGroup1LargeDecayClass5");
	
	/**Detailed substrate - species group 2 large class 1 log initial 
	 * conditions proportion*/
	protected ModelFloat m_fSpGroup2LargeClass1InitLog = new ModelFloat(0,
			"Species Group 2 Large Class 1 Initial Log Prop (0-1)",
			"su_initialLogSpGroup2LargeDecayClass1");
	
	/**Detailed substrate - species group 2 large class 2 log initial 
	 * conditions proportion*/
	protected ModelFloat m_fSpGroup2LargeClass2InitLog = new ModelFloat(0,
			"Species Group 2 Large Class 2 Initial Log Prop (0-1)",
			"su_initialLogSpGroup2LargeDecayClass2");
	
	/**Detailed substrate - species group 2 large class 3 log initial 
	 * conditions proportion*/
	protected ModelFloat m_fSpGroup2LargeClass3InitLog = new ModelFloat(0,
			"Species Group 2 Large Class 3 Initial Log Prop (0-1)",
			"su_initialLogSpGroup2LargeDecayClass3");
	
	/**Detailed substrate - species group 2 large class 4 log initial 
	 * conditions proportion*/
	protected ModelFloat m_fSpGroup2LargeClass4InitLog = new ModelFloat(0,
			"Species Group 2 Large Class 4 Initial Log Prop (0-1)",
			"su_initialLogSpGroup2LargeDecayClass4");
	
	/**Detailed substrate - species group 2 large class 5 log initial 
	 * conditions proportion*/
	protected ModelFloat m_fSpGroup2LargeClass5InitLog = new ModelFloat(0,
			"Species Group 2 Large Class 5 Initial Log Prop (0-1)",
			"su_initialLogSpGroup2LargeDecayClass5");
	
	/**Detailed substrate - species group 3 large class 1 log initial 
	 * conditions proportion*/
	protected ModelFloat m_fSpGroup3LargeClass1InitLog = new ModelFloat(0,
			"Species Group 3 Large Class 1 Initial Log Prop (0-1)",
			"su_initialLogSpGroup3LargeDecayClass1");
	
	/**Detailed substrate - species group 3 large class 2 log initial 
	 * conditions proportion*/
	protected ModelFloat m_fSpGroup3LargeClass2InitLog = new ModelFloat(0,
			"Species Group 3 Large Class 2 Initial Log Prop (0-1)",
			"su_initialLogSpGroup3LargeDecayClass2");
	
	/**Detailed substrate - species group 3 large class 3 log initial 
	 * conditions proportion*/
	protected ModelFloat m_fSpGroup3LargeClass3InitLog = new ModelFloat(0,
			"Species Group 3 Large Class 3 Initial Log Prop (0-1)",
			"su_initialLogSpGroup3LargeDecayClass3");
	
	/**Detailed substrate - species group 3 large class 4 log initial 
	 * conditions proportion*/
	protected ModelFloat m_fSpGroup3LargeClass4InitLog = new ModelFloat(0,
			"Species Group 3 Large Class 4 Initial Log Prop (0-1)",
			"su_initialLogSpGroup3LargeDecayClass4");
	
	/**Detailed substrate - species group 3 large class 5 log initial 
	 * conditions proportion*/
	protected ModelFloat m_fSpGroup3LargeClass5InitLog = new ModelFloat(0,
			"Species Group 3 Large Class 5 Initial Log Prop (0-1)",
			"su_initialLogSpGroup3LargeDecayClass5");
	
	/**Detailed substrate - species group 1 small class 1 partial cut log
	 * proportion*/
	protected ModelFloat m_fSpGroup1SmallClass1PartCutLog = new ModelFloat(0,
			"Species Group 1 Small Class 1 Partial Cut Log (0-1)",
			"su_partialCutLogSpGroup1SmallDecayClass1");
	
	/**Detailed substrate - species group 1 small class 2 partial cut log
	 * proportion*/
	protected ModelFloat m_fSpGroup1SmallClass2PartCutLog = new ModelFloat(0,
			"Species Group 1 Small Class 2 Partial Cut Log (0-1)",
			"su_partialCutLogSpGroup1SmallDecayClass2");
	
	/**Detailed substrate - species group 1 small class 3 partial cut log
	 * proportion*/
	protected ModelFloat m_fSpGroup1SmallClass3PartCutLog = new ModelFloat(0,
			"Species Group 1 Small Class 3 Partial Cut Log (0-1)",
			"su_partialCutLogSpGroup1SmallDecayClass3");
	
	/**Detailed substrate - species group 1 small class 4 partial cut log
	 * proportion*/
	protected ModelFloat m_fSpGroup1SmallClass4PartCutLog = new ModelFloat(0,
			"Species Group 1 Small Class 4 Partial Cut Log (0-1)",
			"su_partialCutLogSpGroup1SmallDecayClass4");
	
	/**Detailed substrate - species group 1 small class 5 partial cut log
	 * proportion*/
	protected ModelFloat m_fSpGroup1SmallClass5PartCutLog = new ModelFloat(0,
			"Species Group 1 Small Class 5 Partial Cut Log (0-1)",
			"su_partialCutLogSpGroup1SmallDecayClass5");
	
	/**Detailed substrate - species group 2 small class 1 partial cut log
	 * proportion*/
	protected ModelFloat m_fSpGroup2SmallClass1PartCutLog = new ModelFloat(0,
			"Species Group 2 Small Class 1 Partial Cut Log (0-1)",
			"su_partialCutLogSpGroup2SmallDecayClass1");
	
	/**Detailed substrate - species group 2 small class 2 partial cut log
	 * proportion*/
	protected ModelFloat m_fSpGroup2SmallClass2PartCutLog = new ModelFloat(0,
			"Species Group 2 Small Class 2 Partial Cut Log (0-1)",
			"su_partialCutLogSpGroup2SmallDecayClass2");
	
	/**Detailed substrate - species group 2 small class 3 partial cut log
	 * proportion*/
	protected ModelFloat m_fSpGroup2SmallClass3PartCutLog = new ModelFloat(0,
			"Species Group 2 Small Class 3 Partial Cut Log (0-1)",
			"su_partialCutLogSpGroup2SmallDecayClass3");
	
	/**Detailed substrate - species group 2 small class 4 partial cut log
	 * proportion*/
	protected ModelFloat m_fSpGroup2SmallClass4PartCutLog = new ModelFloat(0,
			"Species Group 2 Small Class 4 Partial Cut Log (0-1)",
			"su_partialCutLogSpGroup2SmallDecayClass4");
	
	/**Detailed substrate - species group 2 small class 5 partial cut log
	 * proportion*/
	protected ModelFloat m_fSpGroup2SmallClass5PartCutLog = new ModelFloat(0,
			"Species Group 2 Small Class 5 Partial Cut Log (0-1)",
			"su_partialCutLogSpGroup2SmallDecayClass5");
	
	/**Detailed substrate - species group 3 small class 1 partial cut log
	 * proportion*/
	protected ModelFloat m_fSpGroup3SmallClass1PartCutLog = new ModelFloat(0,
			"Species Group 3 Small Class 1 Partial Cut Log (0-1)",
			"su_partialCutLogSpGroup3SmallDecayClass1");
	
	/**Detailed substrate - species group 3 small class 2 partial cut log
	 * proportion*/
	protected ModelFloat m_fSpGroup3SmallClass2PartCutLog = new ModelFloat(0,
			"Species Group 3 Small Class 2 Partial Cut Log (0-1)",
			"su_partialCutLogSpGroup3SmallDecayClass2");
	
	/**Detailed substrate - species group 3 small class 3 partial cut log
	 * proportion*/
	protected ModelFloat m_fSpGroup3SmallClass3PartCutLog = new ModelFloat(0,
			"Species Group 3 Small Class 3 Partial Cut Log (0-1)",
			"su_partialCutLogSpGroup3SmallDecayClass3");
	
	/**Detailed substrate - species group 3 small class 4 partial cut log
	 * proportion*/
	protected ModelFloat m_fSpGroup3SmallClass4PartCutLog = new ModelFloat(0,
			"Species Group 3 Small Class 4 Partial Cut Log (0-1)",
			"su_partialCutLogSpGroup3SmallDecayClass4");
	
	/**Detailed substrate - species group 3 small class 5 partial cut log
	 * proportion*/
	protected ModelFloat m_fSpGroup3SmallClass5PartCutLog = new ModelFloat(0,
			"Species Group 3 Small Class 5 Partial Cut Log (0-1)",
			"su_partialCutLogSpGroup3SmallDecayClass5");
	
	/**Detailed substrate - species group 1 large class 1 partial cut log
	 * proportion*/
	protected ModelFloat m_fSpGroup1LargeClass1PartCutLog = new ModelFloat(0,
			"Species Group 1 Large Class 1 Partial Cut Log (0-1)",
			"su_partialCutLogSpGroup1LargeDecayClass1");
	
	/**Detailed substrate - species group 1 large class 2 partial cut log
	 * proportion*/
	protected ModelFloat m_fSpGroup1LargeClass2PartCutLog = new ModelFloat(0,
			"Species Group 1 Large Class 2 Partial Cut Log (0-1)",
			"su_partialCutLogSpGroup1LargeDecayClass2");
	
	/**Detailed substrate - species group 1 large class 3 partial cut log
	 * proportion*/
	protected ModelFloat m_fSpGroup1LargeClass3PartCutLog = new ModelFloat(0,
			"Species Group 1 Large Class 3 Partial Cut Log (0-1)",
			"su_partialCutLogSpGroup1LargeDecayClass3");
	
	/**Detailed substrate - species group 1 large class 4 partial cut log
	 * proportion*/
	protected ModelFloat m_fSpGroup1LargeClass4PartCutLog = new ModelFloat(0,
			"Species Group 1 Large Class 4 Partial Cut Log (0-1)",
			"su_partialCutLogSpGroup1LargeDecayClass4");
	
	/**Detailed substrate - species group 1 large class 5 partial cut log
	 * proportion*/
	protected ModelFloat m_fSpGroup1LargeClass5PartCutLog = new ModelFloat(0,
			"Species Group 1 Large Class 5 Partial Cut Log (0-1)",
			"su_partialCutLogSpGroup1LargeDecayClass5");
	
	/**Detailed substrate - species group 2 large class 1 partial cut log
	 * proportion*/
	protected ModelFloat m_fSpGroup2LargeClass1PartCutLog = new ModelFloat(0,
			"Species Group 2 Large Class 1 Partial Cut Log (0-1)",
			"su_partialCutLogSpGroup2LargeDecayClass1");
	
	/**Detailed substrate - species group 2 large class 2 partial cut log
	 * proportion*/
	protected ModelFloat m_fSpGroup2LargeClass2PartCutLog = new ModelFloat(0,
			"Species Group 2 Large Class 2 Partial Cut Log (0-1)",
			"su_partialCutLogSpGroup2LargeDecayClass2");
	
	/**Detailed substrate - species group 2 large class 3 partial cut log
	 * proportion*/
	protected ModelFloat m_fSpGroup2LargeClass3PartCutLog = new ModelFloat(0,
			"Species Group 2 Large Class 3 Partial Cut Log (0-1)",
			"su_partialCutLogSpGroup2LargeDecayClass3");
	
	/**Detailed substrate - species group 2 large class 4 partial cut log
	 * proportion*/
	protected ModelFloat m_fSpGroup2LargeClass4PartCutLog = new ModelFloat(0,
			"Species Group 2 Large Class 4 Partial Cut Log (0-1)",
			"su_partialCutLogSpGroup2LargeDecayClass4");
	
	/**Detailed substrate - species group 2 large class 5 partial cut log
	 * proportion*/
	protected ModelFloat m_fSpGroup2LargeClass5PartCutLog = new ModelFloat(0,
			"Species Group 2 Large Class 5 Partial Cut Log (0-1)",
			"su_partialCutLogSpGroup2LargeDecayClass5");
	
	/**Detailed substrate - species group 3 large class 1 partial cut log
	 * proportion*/
	protected ModelFloat m_fSpGroup3LargeClass1PartCutLog = new ModelFloat(0,
			"Species Group 3 Large Class 1 Partial Cut Log (0-1)",
			"su_partialCutLogSpGroup3LargeDecayClass1");
	
	/**Detailed substrate - species group 3 large class 2 partial cut log
	 * proportion*/
	protected ModelFloat m_fSpGroup3LargeClass2PartCutLog = new ModelFloat(0,
			"Species Group 3 Large Class 2 Partial Cut Log (0-1)",
			"su_partialCutLogSpGroup3LargeDecayClass2");
	
	/**Detailed substrate - species group 3 large class 3 partial cut log
	 * proportion*/
	protected ModelFloat m_fSpGroup3LargeClass3PartCutLog = new ModelFloat(0,
			"Species Group 3 Large Class 3 Partial Cut Log (0-1)",
			"su_partialCutLogSpGroup3LargeDecayClass3");
	
	/**Detailed substrate - species group 3 large class 4 partial cut log
	 * proportion*/
	protected ModelFloat m_fSpGroup3LargeClass4PartCutLog = new ModelFloat(0,
			"Species Group 3 Large Class 4 Partial Cut Log (0-1)",
			"su_partialCutLogSpGroup3LargeDecayClass4");
	
	/**Detailed substrate - species group 3 large class 5 partial cut log
	 * proportion*/
	protected ModelFloat m_fSpGroup3LargeClass5PartCutLog = new ModelFloat(0,
			"Species Group 3 Large Class 5 Partial Cut Log (0-1)",
			"su_partialCutLogSpGroup3LargeDecayClass5");
	
	/**Detailed substrate - species group 1 small class 1 gap cut log
	 * proportion*/
	protected ModelFloat m_fSpGroup1SmallClass1GapCutLog = new ModelFloat(0,
			"Species Group 1 Small Class 1 Gap Cut Log (0-1)",
			"su_gapCutLogSpGroup1SmallDecayClass1");
	
	/**Detailed substrate - species group 1 small class 2 gap cut log
	 * proportion*/
	protected ModelFloat m_fSpGroup1SmallClass2GapCutLog = new ModelFloat(0,
			"Species Group 1 Small Class 2 Gap Cut Log (0-1)",
			"su_gapCutLogSpGroup1SmallDecayClass2");
	
	/**Detailed substrate - species group 1 small class 3 gap cut log
	 * proportion*/
	protected ModelFloat m_fSpGroup1SmallClass3GapCutLog = new ModelFloat(0,
			"Species Group 1 Small Class 3 Gap Cut Log (0-1)",
			"su_gapCutLogSpGroup1SmallDecayClass3");
	
	/**Detailed substrate - species group 1 small class 4 gap cut log
	 * proportion*/
	protected ModelFloat m_fSpGroup1SmallClass4GapCutLog = new ModelFloat(0,
			"Species Group 1 Small Class 4 Gap Cut Log (0-1)",
			"su_gapCutLogSpGroup1SmallDecayClass4");
	
	/**Detailed substrate - species group 1 small class 5 gap cut log
	 * proportion*/
	protected ModelFloat m_fSpGroup1SmallClass5GapCutLog = new ModelFloat(0,
			"Species Group 1 Small Class 5 Gap Cut Log (0-1)",
			"su_gapCutLogSpGroup1SmallDecayClass5");
	
	/**Detailed substrate - species group 2 small class 1 gap cut log
	 * proportion*/
	protected ModelFloat m_fSpGroup2SmallClass1GapCutLog = new ModelFloat(0,
			"Species Group 2 Small Class 1 Gap Cut Log (0-1)",
			"su_gapCutLogSpGroup2SmallDecayClass1");
	
	/**Detailed substrate - species group 2 small class 2 gap cut log
	 * proportion*/
	protected ModelFloat m_fSpGroup2SmallClass2GapCutLog = new ModelFloat(0,
			"Species Group 2 Small Class 2 Gap Cut Log (0-1)",
			"su_gapCutLogSpGroup2SmallDecayClass2");
	
	/**Detailed substrate - species group 2 small class 3 gap cut log
	 * proportion*/
	protected ModelFloat m_fSpGroup2SmallClass3GapCutLog = new ModelFloat(0,
			"Species Group 2 Small Class 3 Gap Cut Log (0-1)",
			"su_gapCutLogSpGroup2SmallDecayClass3");
	
	/**Detailed substrate - species group 2 small class 4 gap cut log
	 * proportion*/
	protected ModelFloat m_fSpGroup2SmallClass4GapCutLog = new ModelFloat(0,
			"Species Group 2 Small Class 4 Gap Cut Log (0-1)",
			"su_gapCutLogSpGroup2SmallDecayClass4");
	
	/**Detailed substrate - species group 2 small class 5 gap cut log
	 * proportion*/
	protected ModelFloat m_fSpGroup2SmallClass5GapCutLog = new ModelFloat(0,
			"Species Group 2 Small Class 5 Gap Cut Log (0-1)",
			"su_gapCutLogSpGroup2SmallDecayClass5");
	
	/**Detailed substrate - species group 3 small class 1 gap cut log
	 * proportion*/
	protected ModelFloat m_fSpGroup3SmallClass1GapCutLog = new ModelFloat(0,
			"Species Group 3 Small Class 1 Gap Cut Log (0-1)",
			"su_gapCutLogSpGroup3SmallDecayClass1");
	
	/**Detailed substrate - species group 3 small class 2 gap cut log
	 * proportion*/
	protected ModelFloat m_fSpGroup3SmallClass2GapCutLog = new ModelFloat(0,
			"Species Group 3 Small Class 2 Gap Cut Log (0-1)",
			"su_gapCutLogSpGroup3SmallDecayClass2");
	
	/**Detailed substrate - species group 3 small class 3 gap cut log
	 * proportion*/
	protected ModelFloat m_fSpGroup3SmallClass3GapCutLog = new ModelFloat(0,
			"Species Group 3 Small Class 3 Gap Cut Log (0-1)",
			"su_gapCutLogSpGroup3SmallDecayClass3");
	
	/**Detailed substrate - species group 3 small class 4 gap cut log
	 * proportion*/
	protected ModelFloat m_fSpGroup3SmallClass4GapCutLog = new ModelFloat(0,
			"Species Group 3 Small Class 4 Gap Cut Log (0-1)",
			"su_gapCutLogSpGroup3SmallDecayClass4");
	
	/**Detailed substrate - species group 3 small class 5 gap cut log
	 * proportion*/
	protected ModelFloat m_fSpGroup3SmallClass5GapCutLog = new ModelFloat(0,
			"Species Group 3 Small Class 5 Gap Cut Log (0-1)",
			"su_gapCutLogSpGroup3SmallDecayClass5");
	
	/**Detailed substrate - species group 1 large class 1 gap cut log
	 * proportion*/
	protected ModelFloat m_fSpGroup1LargeClass1GapCutLog = new ModelFloat(0,
			"Species Group 1 Large Class 1 Gap Cut Log (0-1)",
			"su_gapCutLogSpGroup1LargeDecayClass1");
	
	/**Detailed substrate - species group 1 large class 2 gap cut log
	 * proportion*/
	protected ModelFloat m_fSpGroup1LargeClass2GapCutLog = new ModelFloat(0,
			"Species Group 1 Large Class 2 Gap Cut Log (0-1)",
			"su_gapCutLogSpGroup1LargeDecayClass2");
	
	/**Detailed substrate - species group 1 large class 3 gap cut log
	 * proportion*/
	protected ModelFloat m_fSpGroup1LargeClass3GapCutLog = new ModelFloat(0,
			"Species Group 1 Large Class 3 Gap Cut Log (0-1)",
			"su_gapCutLogSpGroup1LargeDecayClass3");
	
	/**Detailed substrate - species group 1 large class 4 gap cut log
	 * proportion*/
	protected ModelFloat m_fSpGroup1LargeClass4GapCutLog = new ModelFloat(0,
			"Species Group 1 Large Class 4 Gap Cut Log (0-1)",
			"su_gapCutLogSpGroup1LargeDecayClass4");
	
	/**Detailed substrate - species group 1 large class 5 gap cut log
	 * proportion*/
	protected ModelFloat m_fSpGroup1LargeClass5GapCutLog = new ModelFloat(0,
			"Species Group 1 Large Class 5 Gap Cut Log (0-1)",
			"su_gapCutLogSpGroup1LargeDecayClass5");
	
	/**Detailed substrate - species group 2 large class 1 gap cut log
	 * proportion*/
	protected ModelFloat m_fSpGroup2LargeClass1GapCutLog = new ModelFloat(0,
			"Species Group 2 Large Class 1 Gap Cut Log (0-1)",
			"su_gapCutLogSpGroup2LargeDecayClass1");
	
	/**Detailed substrate - species group 2 large class 2 gap cut log
	 * proportion*/
	protected ModelFloat m_fSpGroup2LargeClass2GapCutLog = new ModelFloat(0,
			"Species Group 2 Large Class 2 Gap Cut Log (0-1)",
			"su_gapCutLogSpGroup2LargeDecayClass2");
	
	/**Detailed substrate - species group 2 large class 3 gap cut log
	 * proportion*/
	protected ModelFloat m_fSpGroup2LargeClass3GapCutLog = new ModelFloat(0,
			"Species Group 2 Large Class 3 Gap Cut Log (0-1)",
			"su_gapCutLogSpGroup2LargeDecayClass3");
	
	/**Detailed substrate - species group 2 large class 4 gap cut log
	 * proportion*/
	protected ModelFloat m_fSpGroup2LargeClass4GapCutLog = new ModelFloat(0,
			"Species Group 2 Large Class 4 Gap Cut Log (0-1)",
			"su_gapCutLogSpGroup2LargeDecayClass4");
	
	/**Detailed substrate - species group 2 large class 5 gap cut log
	 * proportion*/
	protected ModelFloat m_fSpGroup2LargeClass5GapCutLog = new ModelFloat(0,
			"Species Group 2 Large Class 5 Gap Cut Log (0-1)",
			"su_gapCutLogSpGroup2LargeDecayClass5");
	
	/**Detailed substrate - species group 3 large class 1 gap cut log
	 * proportion*/
	protected ModelFloat m_fSpGroup3LargeClass1GapCutLog = new ModelFloat(0,
			"Species Group 3 Large Class 1 Gap Cut Log (0-1)",
			"su_gapCutLogSpGroup3LargeDecayClass1");
	
	/**Detailed substrate - species group 3 large class 2 gap cut log
	 * proportion*/
	protected ModelFloat m_fSpGroup3LargeClass2GapCutLog = new ModelFloat(0,
			"Species Group 3 Large Class 2 Gap Cut Log (0-1)",
			"su_gapCutLogSpGroup3LargeDecayClass2");
	
	/**Detailed substrate - species group 3 large class 3 gap cut log
	 * proportion*/
	protected ModelFloat m_fSpGroup3LargeClass3GapCutLog = new ModelFloat(0,
			"Species Group 3 Large Class 3 Gap Cut Log (0-1)",
			"su_gapCutLogSpGroup3LargeDecayClass3");
	
	/**Detailed substrate - species group 3 large class 4 gap cut log
	 * proportion*/
	protected ModelFloat m_fSpGroup3LargeClass4GapCutLog = new ModelFloat(0,
			"Species Group 3 Large Class 4 Gap Cut Log (0-1)",
			"su_gapCutLogSpGroup3LargeDecayClass4");
	
	/**Detailed substrate - species group 3 large class 5 gap cut log
	 * proportion*/
	protected ModelFloat m_fSpGroup3LargeClass5GapCutLog = new ModelFloat(0,
			"Species Group 3 Large Class 5 Gap Cut Log (0-1)",
			"su_gapCutLogSpGroup3LargeDecayClass5");
	
	/**Detailed substrate - species group 1 small class 1 clear cut log
	 * proportion*/
	protected ModelFloat m_fSpGroup1SmallClass1ClearCutLog = new ModelFloat(0,
			"Species Group 1 Small Class 1 Clear Cut Log (0-1)",
			"su_clearCutLogSpGroup1SmallDecayClass1");
	
	/**Detailed substrate - species group 1 small class 2 clear cut log
	 * proportion*/
	protected ModelFloat m_fSpGroup1SmallClass2ClearCutLog = new ModelFloat(0,
			"Species Group 1 Small Class 2 Clear Cut Log (0-1)",
			"su_clearCutLogSpGroup1SmallDecayClass2");
	
	/**Detailed substrate - species group 1 small class 3 clear cut log
	 * proportion*/
	protected ModelFloat m_fSpGroup1SmallClass3ClearCutLog = new ModelFloat(0,
			"Species Group 1 Small Class 3 Clear Cut Log (0-1)",
			"su_clearCutLogSpGroup1SmallDecayClass3");
	
	/**Detailed substrate - species group 1 small class 4 clear cut log
	 * proportion*/
	protected ModelFloat m_fSpGroup1SmallClass4ClearCutLog = new ModelFloat(0,
			"Species Group 1 Small Class 4 Clear Cut Log (0-1)",
			"su_clearCutLogSpGroup1SmallDecayClass4");
	
	/**Detailed substrate - species group 1 small class 5 clear cut log
	 * proportion*/
	protected ModelFloat m_fSpGroup1SmallClass5ClearCutLog = new ModelFloat(0,
			"Species Group 1 Small Class 5 Clear Cut Log (0-1)",
			"su_clearCutLogSpGroup1SmallDecayClass5");
	
	/**Detailed substrate - species group 2 small class 1 clear cut log
	 * proportion*/
	protected ModelFloat m_fSpGroup2SmallClass1ClearCutLog = new ModelFloat(0,
			"Species Group 2 Small Class 1 Clear Cut Log (0-1)",
			"su_clearCutLogSpGroup2SmallDecayClass1");
	
	/**Detailed substrate - species group 2 small class 2 clear cut log
	 * proportion*/
	protected ModelFloat m_fSpGroup2SmallClass2ClearCutLog = new ModelFloat(0,
			"Species Group 2 Small Class 2 Clear Cut Log (0-1)",
			"su_clearCutLogSpGroup2SmallDecayClass2");
	
	/**Detailed substrate - species group 2 small class 3 clear cut log
	 * proportion*/
	protected ModelFloat m_fSpGroup2SmallClass3ClearCutLog = new ModelFloat(0,
			"Species Group 2 Small Class 3 Clear Cut Log (0-1)",
			"su_clearCutLogSpGroup2SmallDecayClass3");
	
	/**Detailed substrate - species group 2 small class 4 clear cut log
	 * proportion*/
	protected ModelFloat m_fSpGroup2SmallClass4ClearCutLog = new ModelFloat(0,
			"Species Group 2 Small Class 4 Clear Cut Log (0-1)",
			"su_clearCutLogSpGroup2SmallDecayClass4");
	
	/**Detailed substrate - species group 2 small class 5 clear cut log
	 * proportion*/
	protected ModelFloat m_fSpGroup2SmallClass5ClearCutLog = new ModelFloat(0,
			"Species Group 2 Small Class 5 Clear Cut Log (0-1)",
			"su_clearCutLogSpGroup2SmallDecayClass5");
	
	/**Detailed substrate - species group 3 small class 1 clear cut log
	 * proportion*/
	protected ModelFloat m_fSpGroup3SmallClass1ClearCutLog = new ModelFloat(0,
			"Species Group 3 Small Class 1 Clear Cut Log (0-1)",
			"su_clearCutLogSpGroup3SmallDecayClass1");
	
	/**Detailed substrate - species group 3 small class 2 clear cut log
	 * proportion*/
	protected ModelFloat m_fSpGroup3SmallClass2ClearCutLog = new ModelFloat(0,
			"Species Group 3 Small Class 2 Clear Cut Log (0-1)",
			"su_clearCutLogSpGroup3SmallDecayClass2");
	
	/**Detailed substrate - species group 3 small class 3 clear cut log
	 * proportion*/
	protected ModelFloat m_fSpGroup3SmallClass3ClearCutLog = new ModelFloat(0,
			"Species Group 3 Small Class 3 Clear Cut Log (0-1)",
			"su_clearCutLogSpGroup3SmallDecayClass3");
	
	/**Detailed substrate - species group 3 small class 4 clear cut log
	 * proportion*/
	protected ModelFloat m_fSpGroup3SmallClass4ClearCutLog = new ModelFloat(0,
			"Species Group 3 Small Class 4 Clear Cut Log (0-1)",
			"su_clearCutLogSpGroup3SmallDecayClass4");
	
	/**Detailed substrate - species group 3 small class 5 clear cut log
	 * proportion*/
	protected ModelFloat m_fSpGroup3SmallClass5ClearCutLog = new ModelFloat(0,
			"Species Group 3 Small Class 5 Clear Cut Log (0-1)",
			"su_clearCutLogSpGroup3SmallDecayClass5");
	
	/**Detailed substrate - species group 1 large class 1 clear cut log
	 * proportion*/
	protected ModelFloat m_fSpGroup1LargeClass1ClearCutLog = new ModelFloat(0,
			"Species Group 1 Large Class 1 Clear Cut Log (0-1)",
			"su_clearCutLogSpGroup1LargeDecayClass1");
	
	/**Detailed substrate - species group 1 large class 2 clear cut log
	 * proportion*/
	protected ModelFloat m_fSpGroup1LargeClass2ClearCutLog = new ModelFloat(0,
			"Species Group 1 Large Class 2 Clear Cut Log (0-1)",
			"su_clearCutLogSpGroup1LargeDecayClass2");
	
	/**Detailed substrate - species group 1 large class 3 clear cut log
	 * proportion*/
	protected ModelFloat m_fSpGroup1LargeClass3ClearCutLog = new ModelFloat(0,
			"Species Group 1 Large Class 3 Clear Cut Log (0-1)",
			"su_clearCutLogSpGroup1LargeDecayClass3");
	
	/**Detailed substrate - species group 1 large class 4 clear cut log
	 * proportion*/
	protected ModelFloat m_fSpGroup1LargeClass4ClearCutLog = new ModelFloat(0,
			"Species Group 1 Large Class 4 Clear Cut Log (0-1)",
			"su_clearCutLogSpGroup1LargeDecayClass4");
	
	/**Detailed substrate - species group 1 large class 5 clear cut log
	 * proportion*/
	protected ModelFloat m_fSpGroup1LargeClass5ClearCutLog = new ModelFloat(0,
			"Species Group 1 Large Class 5 Clear Cut Log (0-1)",
			"su_clearCutLogSpGroup1LargeDecayClass5");
	
	/**Detailed substrate - species group 2 large class 1 clear cut log
	 * proportion*/
	protected ModelFloat m_fSpGroup2LargeClass1ClearCutLog = new ModelFloat(0,
			"Species Group 2 Large Class 1 Clear Cut Log (0-1)",
			"su_clearCutLogSpGroup2LargeDecayClass1");
	
	/**Detailed substrate - species group 2 large class 2 clear cut log
	 * proportion*/
	protected ModelFloat m_fSpGroup2LargeClass2ClearCutLog = new ModelFloat(0,
			"Species Group 2 Large Class 2 Clear Cut Log (0-1)",
			"su_clearCutLogSpGroup2LargeDecayClass2");
	
	/**Detailed substrate - species group 2 large class 3 clear cut log
	 * proportion*/
	protected ModelFloat m_fSpGroup2LargeClass3ClearCutLog = new ModelFloat(0,
			"Species Group 2 Large Class 3 Clear Cut Log (0-1)",
			"su_clearCutLogSpGroup2LargeDecayClass3");
	
	/**Detailed substrate - species group 2 large class 4 clear cut log
	 * proportion*/
	protected ModelFloat m_fSpGroup2LargeClass4ClearCutLog = new ModelFloat(0,
			"Species Group 2 Large Class 4 Clear Cut Log (0-1)",
			"su_clearCutLogSpGroup2LargeDecayClass4");
	
	/**Detailed substrate - species group 2 large class 5 clear cut log
	 * proportion*/
	protected ModelFloat m_fSpGroup2LargeClass5ClearCutLog = new ModelFloat(0,
			"Species Group 2 Large Class 5 Clear Cut Log (0-1)",
			"su_clearCutLogSpGroup2LargeDecayClass5");
	
	/**Detailed substrate - species group 3 large class 1 clear cut log
	 * proportion*/
	protected ModelFloat m_fSpGroup3LargeClass1ClearCutLog = new ModelFloat(0,
			"Species Group 3 Large Class 1 Clear Cut Log (0-1)",
			"su_clearCutLogSpGroup3LargeDecayClass1");
	
	/**Detailed substrate - species group 3 large class 2 clear cut log
	 * proportion*/
	protected ModelFloat m_fSpGroup3LargeClass2ClearCutLog = new ModelFloat(0,
			"Species Group 3 Large Class 2 Clear Cut Log (0-1)",
			"su_clearCutLogSpGroup3LargeDecayClass2");
	
	/**Detailed substrate - species group 3 large class 3 clear cut log
	 * proportion*/
	protected ModelFloat m_fSpGroup3LargeClass3ClearCutLog = new ModelFloat(0,
			"Species Group 3 Large Class 3 Clear Cut Log (0-1)",
			"su_clearCutLogSpGroup3LargeDecayClass3");
	
	/**Detailed substrate - species group 3 large class 4 clear cut log
	 * proportion*/
	protected ModelFloat m_fSpGroup3LargeClass4ClearCutLog = new ModelFloat(0,
			"Species Group 3 Large Class 4 Clear Cut Log (0-1)",
			"su_clearCutLogSpGroup3LargeDecayClass4");
	
	/**Detailed substrate - species group 3 large class 5 clear cut log
	 * proportion*/
	protected ModelFloat m_fSpGroup3LargeClass5ClearCutLog = new ModelFloat(0,
			"Species Group 3 Large Class 5 Clear Cut Log (0-1)",
			"su_clearCutLogSpGroup3LargeDecayClass5");
	
	/**Detailed substrate - mean diameter of small logs at initial conditions*/
	protected ModelFloat m_fInitSmallLogMeanDiam = new ModelFloat(0,
			"Initial Small Logs Mean Diameter (cm)", "su_initialSmallLogMeanDiameter");
	
	/**Detailed substrate - mean diameter of large logs at initial conditions*/
	protected ModelFloat m_fInitLargeLogMeanDiam = new ModelFloat(0,
			"Initial Large Logs Mean Diameter (cm)", "su_initialLargeLogMeanDiameter");
	
	/**Detailed substrate - mean diameter of small logs for partial cuts*/
	protected ModelFloat m_fPartCutSmallLogMeanDiam = new ModelFloat(0,
			"Partial Cut Small Logs Mean Diameter (cm)", 
			"su_partialCutSmallLogMeanDiameter");
	
	/**Detailed substrate - mean diameter of large logs for partial cuts*/
	protected ModelFloat m_fPartCutLargeLogMeanDiam = new ModelFloat(0,
			"Partial Cut Large Logs Mean Diameter (cm)", 
			"su_partialCutLargeLogMeanDiameter");
	
	/**Detailed substrate - mean diameter of small logs for gap cuts*/
	protected ModelFloat m_fGapCutSmallLogMeanDiam = new ModelFloat(0,
			"Gap Cut Small Logs Mean Diameter (cm)", "su_gapCutSmallLogMeanDiameter");
	
	/**Detailed substrate - mean diameter of large logs for gap cuts*/
	protected ModelFloat m_fGapCutLargeLogMeanDiam = new ModelFloat(0,
			"Gap Cut Large Logs Mean Diameter (cm)", "su_gapCutLargeLogMeanDiameter");
	
	/**Detailed substrate - mean diameter of small logs for clear cuts*/
	protected ModelFloat m_fClearCutSmallLogMeanDiam = new ModelFloat(0,
			"Clear Cut Small Logs Mean Diameter (cm)", "su_clearCutSmallLogMeanDiameter");
	
	/**Detailed substrate - mean diameter of large logs for clear cuts*/
	protected ModelFloat m_fClearCutLargeLogMeanDiam = new ModelFloat(0,
			"Clear Cut Large Logs Mean Diameter (cm)", "su_clearCutLargeLogMeanDiameter");

  /**
   * Constructor
   * @param oManager GUIManager object.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  public SubstrateBehaviors(GUIManager oManager) {
    super(oManager, "Substrate", "substrate");

    m_sHelpID = "data.substrate_behaviors";

    //Set up our child behavior vector
    mp_oChildBehaviors = new Behavior[2];
    
    //Substrate behavior
    mp_oChildBehaviors[0] = new Behavior("substrate", "Substrate", "substrate",
                                         (float) 2.1);
    //Substrate cannot apply to seedlings
    mp_oChildBehaviors[0].SetCanApplyTo(TreePopulation.SEEDLING, false);

    mp_oChildBehaviors[0].AddRequiredData(m_iMaxDecayTime);
    mp_oChildBehaviors[0].AddRequiredData(m_fScarSoilA);
    mp_oChildBehaviors[0].AddRequiredData(m_fScarSoilB);
    mp_oChildBehaviors[0].AddRequiredData(m_fTipUpA);
    mp_oChildBehaviors[0].AddRequiredData(m_fTipUpB);
    mp_oChildBehaviors[0].AddRequiredData(m_fFreshlogA);
    mp_oChildBehaviors[0].AddRequiredData(m_fFreshlogB);
    mp_oChildBehaviors[0].AddRequiredData(m_fDecayedlogA);
    mp_oChildBehaviors[0].AddRequiredData(m_fDecayedlogB);
    mp_oChildBehaviors[0].AddRequiredData(m_fMossProportion);
    mp_oChildBehaviors[0].AddRequiredData(m_fRootTipupFactor);
    mp_oChildBehaviors[0].AddRequiredData(m_iDirectionalTreeFall);
    mp_oChildBehaviors[0].AddRequiredData(m_fInitCondScarSoil);
    mp_oChildBehaviors[0].AddRequiredData(m_fInitCondTipup);
    mp_oChildBehaviors[0].AddRequiredData(m_fInitCondFreshLog);
    mp_oChildBehaviors[0].AddRequiredData(m_fInitCondDecLog);
    mp_oChildBehaviors[0].AddRequiredData(m_fPartialCutScarSoil);
    mp_oChildBehaviors[0].AddRequiredData(m_fPartialCutTipup);
    mp_oChildBehaviors[0].AddRequiredData(m_fPartialCutFreshLog);
    mp_oChildBehaviors[0].AddRequiredData(m_fPartialCutDecLog);
    mp_oChildBehaviors[0].AddRequiredData(m_fGapCutScarSoil);
    mp_oChildBehaviors[0].AddRequiredData(m_fGapCutTipup);
    mp_oChildBehaviors[0].AddRequiredData(m_fGapCutFreshLog);
    mp_oChildBehaviors[0].AddRequiredData(m_fGapCutDecLog);
    mp_oChildBehaviors[0].AddRequiredData(m_fClearCutScarSoil);
    mp_oChildBehaviors[0].AddRequiredData(m_fClearCutTipup);
    mp_oChildBehaviors[0].AddRequiredData(m_fClearCutFreshLog);
    mp_oChildBehaviors[0].AddRequiredData(m_fClearCutDecLog);
    mp_oChildBehaviors[0].AddRequiredData(mp_fProportionOfDeadThatFall);
    mp_oChildBehaviors[0].AddRequiredData(mp_fProportionOfFallenThatUproot);
    mp_oChildBehaviors[0].AddRequiredData(mp_fProportionOfSnagsThatUproot);
    
    //Detailed substrate behavior
    mp_oChildBehaviors[1] = new Behavior("Detailed Substrate", 
    		"Detailed Substrate", "detailedsubstrate", (float) 1.0);
    //Substrate cannot apply to seedlings
    mp_oChildBehaviors[1].SetCanApplyTo(TreePopulation.SEEDLING, false);
    mp_oChildBehaviors[1].SetCanApplyTo(TreePopulation.SAPLING, false);
    
    mp_oChildBehaviors[1].AddRequiredData(mp_iSpeciesGroup);
    mp_oChildBehaviors[1].AddRequiredData(mp_fProportionOfFallenThatUproot);
    mp_oChildBehaviors[1].AddRequiredData(mp_fProportionOfSnagsThatUproot);
    mp_oChildBehaviors[1].AddRequiredData(m_fRootTipupFactor);
    mp_oChildBehaviors[1].AddRequiredData(m_fScarSoilA);
    mp_oChildBehaviors[1].AddRequiredData(m_fScarSoilB);
    mp_oChildBehaviors[1].AddRequiredData(m_fTipUpA);
    mp_oChildBehaviors[1].AddRequiredData(m_fTipUpB);
    mp_oChildBehaviors[1].AddRequiredData(m_iMaxDecayTime);
    mp_oChildBehaviors[1].AddRequiredData(m_fMossProportion);
    mp_oChildBehaviors[1].AddRequiredData(m_iDirectionalTreeFall);
    mp_oChildBehaviors[1].AddRequiredData(m_fInitCondScarSoil);
    mp_oChildBehaviors[1].AddRequiredData(m_fInitCondTipup);
    mp_oChildBehaviors[1].AddRequiredData(m_fLogSizeClassBoundary);
    mp_oChildBehaviors[1].AddRequiredData(m_fPropFallenTreesDecayClass1);
    mp_oChildBehaviors[1].AddRequiredData(m_fPropFallenTreesDecayClass2);
    mp_oChildBehaviors[1].AddRequiredData(m_fPropFallenTreesDecayClass3);
    mp_oChildBehaviors[1].AddRequiredData(m_fPropFallenTreesDecayClass4);
    mp_oChildBehaviors[1].AddRequiredData(m_fPropFallenTreesDecayClass5);
    mp_oChildBehaviors[1].AddRequiredData(m_fPropFallenSnagsDecayClass1);
    mp_oChildBehaviors[1].AddRequiredData(m_fPropFallenSnagsDecayClass2);
    mp_oChildBehaviors[1].AddRequiredData(m_fPropFallenSnagsDecayClass3);
    mp_oChildBehaviors[1].AddRequiredData(m_fPropFallenSnagsDecayClass4);
    mp_oChildBehaviors[1].AddRequiredData(m_fPropFallenSnagsDecayClass5);
    mp_oChildBehaviors[1].AddRequiredData(m_fSpGroup1SmallClass1Alpha);
    mp_oChildBehaviors[1].AddRequiredData(m_fSpGroup1SmallClass2Alpha);
    mp_oChildBehaviors[1].AddRequiredData(m_fSpGroup1SmallClass3Alpha);
    mp_oChildBehaviors[1].AddRequiredData(m_fSpGroup1SmallClass4Alpha);
    mp_oChildBehaviors[1].AddRequiredData(m_fSpGroup1SmallClass5Alpha);
    mp_oChildBehaviors[1].AddRequiredData(m_fSpGroup2SmallClass1Alpha);
    mp_oChildBehaviors[1].AddRequiredData(m_fSpGroup2SmallClass2Alpha);
    mp_oChildBehaviors[1].AddRequiredData(m_fSpGroup2SmallClass3Alpha);
    mp_oChildBehaviors[1].AddRequiredData(m_fSpGroup2SmallClass4Alpha);
    mp_oChildBehaviors[1].AddRequiredData(m_fSpGroup2SmallClass5Alpha);
    mp_oChildBehaviors[1].AddRequiredData(m_fSpGroup3SmallClass1Alpha);
    mp_oChildBehaviors[1].AddRequiredData(m_fSpGroup3SmallClass2Alpha);
    mp_oChildBehaviors[1].AddRequiredData(m_fSpGroup3SmallClass3Alpha);
    mp_oChildBehaviors[1].AddRequiredData(m_fSpGroup3SmallClass4Alpha);
    mp_oChildBehaviors[1].AddRequiredData(m_fSpGroup3SmallClass5Alpha);
    mp_oChildBehaviors[1].AddRequiredData(m_fSpGroup1LargeClass1Alpha);
    mp_oChildBehaviors[1].AddRequiredData(m_fSpGroup1LargeClass2Alpha);
    mp_oChildBehaviors[1].AddRequiredData(m_fSpGroup1LargeClass3Alpha);
    mp_oChildBehaviors[1].AddRequiredData(m_fSpGroup1LargeClass4Alpha);
    mp_oChildBehaviors[1].AddRequiredData(m_fSpGroup1LargeClass5Alpha);
    mp_oChildBehaviors[1].AddRequiredData(m_fSpGroup2LargeClass1Alpha);
    mp_oChildBehaviors[1].AddRequiredData(m_fSpGroup2LargeClass2Alpha);
    mp_oChildBehaviors[1].AddRequiredData(m_fSpGroup2LargeClass3Alpha);
    mp_oChildBehaviors[1].AddRequiredData(m_fSpGroup2LargeClass4Alpha);
    mp_oChildBehaviors[1].AddRequiredData(m_fSpGroup2LargeClass5Alpha);
    mp_oChildBehaviors[1].AddRequiredData(m_fSpGroup3LargeClass1Alpha);
    mp_oChildBehaviors[1].AddRequiredData(m_fSpGroup3LargeClass2Alpha);
    mp_oChildBehaviors[1].AddRequiredData(m_fSpGroup3LargeClass3Alpha);
    mp_oChildBehaviors[1].AddRequiredData(m_fSpGroup3LargeClass4Alpha);
    mp_oChildBehaviors[1].AddRequiredData(m_fSpGroup3LargeClass5Alpha);
    mp_oChildBehaviors[1].AddRequiredData(m_fSpGroup1SmallClass1Beta);
    mp_oChildBehaviors[1].AddRequiredData(m_fSpGroup1SmallClass2Beta);
    mp_oChildBehaviors[1].AddRequiredData(m_fSpGroup1SmallClass3Beta);
    mp_oChildBehaviors[1].AddRequiredData(m_fSpGroup1SmallClass4Beta);
    mp_oChildBehaviors[1].AddRequiredData(m_fSpGroup1SmallClass5Beta);
    mp_oChildBehaviors[1].AddRequiredData(m_fSpGroup2SmallClass1Beta);
    mp_oChildBehaviors[1].AddRequiredData(m_fSpGroup2SmallClass2Beta);
    mp_oChildBehaviors[1].AddRequiredData(m_fSpGroup2SmallClass3Beta);
    mp_oChildBehaviors[1].AddRequiredData(m_fSpGroup2SmallClass4Beta);
    mp_oChildBehaviors[1].AddRequiredData(m_fSpGroup2SmallClass5Beta);
    mp_oChildBehaviors[1].AddRequiredData(m_fSpGroup3SmallClass1Beta);
    mp_oChildBehaviors[1].AddRequiredData(m_fSpGroup3SmallClass2Beta);
    mp_oChildBehaviors[1].AddRequiredData(m_fSpGroup3SmallClass3Beta);
    mp_oChildBehaviors[1].AddRequiredData(m_fSpGroup3SmallClass4Beta);
    mp_oChildBehaviors[1].AddRequiredData(m_fSpGroup3SmallClass5Beta);
    mp_oChildBehaviors[1].AddRequiredData(m_fSpGroup1LargeClass1Beta);
    mp_oChildBehaviors[1].AddRequiredData(m_fSpGroup1LargeClass2Beta);
    mp_oChildBehaviors[1].AddRequiredData(m_fSpGroup1LargeClass3Beta);
    mp_oChildBehaviors[1].AddRequiredData(m_fSpGroup1LargeClass4Beta);
    mp_oChildBehaviors[1].AddRequiredData(m_fSpGroup1LargeClass5Beta);
    mp_oChildBehaviors[1].AddRequiredData(m_fSpGroup2LargeClass1Beta);
    mp_oChildBehaviors[1].AddRequiredData(m_fSpGroup2LargeClass2Beta);
    mp_oChildBehaviors[1].AddRequiredData(m_fSpGroup2LargeClass3Beta);
    mp_oChildBehaviors[1].AddRequiredData(m_fSpGroup2LargeClass4Beta);
    mp_oChildBehaviors[1].AddRequiredData(m_fSpGroup2LargeClass5Beta);
    mp_oChildBehaviors[1].AddRequiredData(m_fSpGroup3LargeClass1Beta);
    mp_oChildBehaviors[1].AddRequiredData(m_fSpGroup3LargeClass2Beta);
    mp_oChildBehaviors[1].AddRequiredData(m_fSpGroup3LargeClass3Beta);
    mp_oChildBehaviors[1].AddRequiredData(m_fSpGroup3LargeClass4Beta);
    mp_oChildBehaviors[1].AddRequiredData(m_fSpGroup3LargeClass5Beta);
    mp_oChildBehaviors[1].AddRequiredData(m_fInitSmallLogMeanDiam);
    mp_oChildBehaviors[1].AddRequiredData(m_fInitLargeLogMeanDiam);
    mp_oChildBehaviors[1].AddRequiredData(m_fSpGroup1SmallClass1InitLog);
    mp_oChildBehaviors[1].AddRequiredData(m_fSpGroup1SmallClass2InitLog);
    mp_oChildBehaviors[1].AddRequiredData(m_fSpGroup1SmallClass3InitLog);
    mp_oChildBehaviors[1].AddRequiredData(m_fSpGroup1SmallClass4InitLog);
    mp_oChildBehaviors[1].AddRequiredData(m_fSpGroup1SmallClass5InitLog);
    mp_oChildBehaviors[1].AddRequiredData(m_fSpGroup2SmallClass1InitLog);
    mp_oChildBehaviors[1].AddRequiredData(m_fSpGroup2SmallClass2InitLog);
    mp_oChildBehaviors[1].AddRequiredData(m_fSpGroup2SmallClass3InitLog);
    mp_oChildBehaviors[1].AddRequiredData(m_fSpGroup2SmallClass4InitLog);
    mp_oChildBehaviors[1].AddRequiredData(m_fSpGroup2SmallClass5InitLog);
    mp_oChildBehaviors[1].AddRequiredData(m_fSpGroup3SmallClass1InitLog);
    mp_oChildBehaviors[1].AddRequiredData(m_fSpGroup3SmallClass2InitLog);
    mp_oChildBehaviors[1].AddRequiredData(m_fSpGroup3SmallClass3InitLog);
    mp_oChildBehaviors[1].AddRequiredData(m_fSpGroup3SmallClass4InitLog);
    mp_oChildBehaviors[1].AddRequiredData(m_fSpGroup3SmallClass5InitLog);
    mp_oChildBehaviors[1].AddRequiredData(m_fSpGroup1LargeClass1InitLog);
    mp_oChildBehaviors[1].AddRequiredData(m_fSpGroup1LargeClass2InitLog);
    mp_oChildBehaviors[1].AddRequiredData(m_fSpGroup1LargeClass3InitLog);
    mp_oChildBehaviors[1].AddRequiredData(m_fSpGroup1LargeClass4InitLog);
    mp_oChildBehaviors[1].AddRequiredData(m_fSpGroup1LargeClass5InitLog);
    mp_oChildBehaviors[1].AddRequiredData(m_fSpGroup2LargeClass1InitLog);
    mp_oChildBehaviors[1].AddRequiredData(m_fSpGroup2LargeClass2InitLog);
    mp_oChildBehaviors[1].AddRequiredData(m_fSpGroup2LargeClass3InitLog);
    mp_oChildBehaviors[1].AddRequiredData(m_fSpGroup2LargeClass4InitLog);
    mp_oChildBehaviors[1].AddRequiredData(m_fSpGroup2LargeClass5InitLog);
    mp_oChildBehaviors[1].AddRequiredData(m_fSpGroup3LargeClass1InitLog);
    mp_oChildBehaviors[1].AddRequiredData(m_fSpGroup3LargeClass2InitLog);
    mp_oChildBehaviors[1].AddRequiredData(m_fSpGroup3LargeClass3InitLog);
    mp_oChildBehaviors[1].AddRequiredData(m_fSpGroup3LargeClass4InitLog);
    mp_oChildBehaviors[1].AddRequiredData(m_fSpGroup3LargeClass5InitLog);
    mp_oChildBehaviors[1].AddRequiredData(m_fPartialCutScarSoil);
    mp_oChildBehaviors[1].AddRequiredData(m_fPartialCutTipup);
    mp_oChildBehaviors[1].AddRequiredData(m_fSpGroup1SmallClass1PartCutLog);
    mp_oChildBehaviors[1].AddRequiredData(m_fSpGroup1SmallClass2PartCutLog);
    mp_oChildBehaviors[1].AddRequiredData(m_fSpGroup1SmallClass3PartCutLog);
    mp_oChildBehaviors[1].AddRequiredData(m_fSpGroup1SmallClass4PartCutLog);
    mp_oChildBehaviors[1].AddRequiredData(m_fSpGroup1SmallClass5PartCutLog);
    mp_oChildBehaviors[1].AddRequiredData(m_fSpGroup2SmallClass1PartCutLog);
    mp_oChildBehaviors[1].AddRequiredData(m_fSpGroup2SmallClass2PartCutLog);
    mp_oChildBehaviors[1].AddRequiredData(m_fSpGroup2SmallClass3PartCutLog);
    mp_oChildBehaviors[1].AddRequiredData(m_fSpGroup2SmallClass4PartCutLog);
    mp_oChildBehaviors[1].AddRequiredData(m_fSpGroup2SmallClass5PartCutLog);
    mp_oChildBehaviors[1].AddRequiredData(m_fSpGroup3SmallClass1PartCutLog);
    mp_oChildBehaviors[1].AddRequiredData(m_fSpGroup3SmallClass2PartCutLog);
    mp_oChildBehaviors[1].AddRequiredData(m_fSpGroup3SmallClass3PartCutLog);
    mp_oChildBehaviors[1].AddRequiredData(m_fSpGroup3SmallClass4PartCutLog);
    mp_oChildBehaviors[1].AddRequiredData(m_fSpGroup3SmallClass5PartCutLog);
    mp_oChildBehaviors[1].AddRequiredData(m_fSpGroup1LargeClass1PartCutLog);
    mp_oChildBehaviors[1].AddRequiredData(m_fSpGroup1LargeClass2PartCutLog);
    mp_oChildBehaviors[1].AddRequiredData(m_fSpGroup1LargeClass3PartCutLog);
    mp_oChildBehaviors[1].AddRequiredData(m_fSpGroup1LargeClass4PartCutLog);
    mp_oChildBehaviors[1].AddRequiredData(m_fSpGroup1LargeClass5PartCutLog);
    mp_oChildBehaviors[1].AddRequiredData(m_fSpGroup2LargeClass1PartCutLog);
    mp_oChildBehaviors[1].AddRequiredData(m_fSpGroup2LargeClass2PartCutLog);
    mp_oChildBehaviors[1].AddRequiredData(m_fSpGroup2LargeClass3PartCutLog);
    mp_oChildBehaviors[1].AddRequiredData(m_fSpGroup2LargeClass4PartCutLog);
    mp_oChildBehaviors[1].AddRequiredData(m_fSpGroup2LargeClass5PartCutLog);
    mp_oChildBehaviors[1].AddRequiredData(m_fSpGroup3LargeClass1PartCutLog);
    mp_oChildBehaviors[1].AddRequiredData(m_fSpGroup3LargeClass2PartCutLog);
    mp_oChildBehaviors[1].AddRequiredData(m_fSpGroup3LargeClass3PartCutLog);
    mp_oChildBehaviors[1].AddRequiredData(m_fSpGroup3LargeClass4PartCutLog);
    mp_oChildBehaviors[1].AddRequiredData(m_fSpGroup3LargeClass5PartCutLog);
    mp_oChildBehaviors[1].AddRequiredData(m_fGapCutScarSoil);
    mp_oChildBehaviors[1].AddRequiredData(m_fGapCutTipup);
    mp_oChildBehaviors[1].AddRequiredData(m_fSpGroup1SmallClass1GapCutLog);
    mp_oChildBehaviors[1].AddRequiredData(m_fSpGroup1SmallClass2GapCutLog);
    mp_oChildBehaviors[1].AddRequiredData(m_fSpGroup1SmallClass3GapCutLog);
    mp_oChildBehaviors[1].AddRequiredData(m_fSpGroup1SmallClass4GapCutLog);
    mp_oChildBehaviors[1].AddRequiredData(m_fSpGroup1SmallClass5GapCutLog);
    mp_oChildBehaviors[1].AddRequiredData(m_fSpGroup2SmallClass1GapCutLog);
    mp_oChildBehaviors[1].AddRequiredData(m_fSpGroup2SmallClass2GapCutLog);
    mp_oChildBehaviors[1].AddRequiredData(m_fSpGroup2SmallClass3GapCutLog);
    mp_oChildBehaviors[1].AddRequiredData(m_fSpGroup2SmallClass4GapCutLog);
    mp_oChildBehaviors[1].AddRequiredData(m_fSpGroup2SmallClass5GapCutLog);
    mp_oChildBehaviors[1].AddRequiredData(m_fSpGroup3SmallClass1GapCutLog);
    mp_oChildBehaviors[1].AddRequiredData(m_fSpGroup3SmallClass2GapCutLog);
    mp_oChildBehaviors[1].AddRequiredData(m_fSpGroup3SmallClass3GapCutLog);
    mp_oChildBehaviors[1].AddRequiredData(m_fSpGroup3SmallClass4GapCutLog);
    mp_oChildBehaviors[1].AddRequiredData(m_fSpGroup3SmallClass5GapCutLog);
    mp_oChildBehaviors[1].AddRequiredData(m_fSpGroup1LargeClass1GapCutLog);
    mp_oChildBehaviors[1].AddRequiredData(m_fSpGroup1LargeClass2GapCutLog);
    mp_oChildBehaviors[1].AddRequiredData(m_fSpGroup1LargeClass3GapCutLog);
    mp_oChildBehaviors[1].AddRequiredData(m_fSpGroup1LargeClass4GapCutLog);
    mp_oChildBehaviors[1].AddRequiredData(m_fSpGroup1LargeClass5GapCutLog);
    mp_oChildBehaviors[1].AddRequiredData(m_fSpGroup2LargeClass1GapCutLog);
    mp_oChildBehaviors[1].AddRequiredData(m_fSpGroup2LargeClass2GapCutLog);
    mp_oChildBehaviors[1].AddRequiredData(m_fSpGroup2LargeClass3GapCutLog);
    mp_oChildBehaviors[1].AddRequiredData(m_fSpGroup2LargeClass4GapCutLog);
    mp_oChildBehaviors[1].AddRequiredData(m_fSpGroup2LargeClass5GapCutLog);
    mp_oChildBehaviors[1].AddRequiredData(m_fSpGroup3LargeClass1GapCutLog);
    mp_oChildBehaviors[1].AddRequiredData(m_fSpGroup3LargeClass2GapCutLog);
    mp_oChildBehaviors[1].AddRequiredData(m_fSpGroup3LargeClass3GapCutLog);
    mp_oChildBehaviors[1].AddRequiredData(m_fSpGroup3LargeClass4GapCutLog);
    mp_oChildBehaviors[1].AddRequiredData(m_fSpGroup3LargeClass5GapCutLog);
    mp_oChildBehaviors[1].AddRequiredData(m_fClearCutScarSoil);
    mp_oChildBehaviors[1].AddRequiredData(m_fClearCutTipup);
    mp_oChildBehaviors[1].AddRequiredData(m_fSpGroup1SmallClass1ClearCutLog);
    mp_oChildBehaviors[1].AddRequiredData(m_fSpGroup1SmallClass2ClearCutLog);
    mp_oChildBehaviors[1].AddRequiredData(m_fSpGroup1SmallClass3ClearCutLog);
    mp_oChildBehaviors[1].AddRequiredData(m_fSpGroup1SmallClass4ClearCutLog);
    mp_oChildBehaviors[1].AddRequiredData(m_fSpGroup1SmallClass5ClearCutLog);
    mp_oChildBehaviors[1].AddRequiredData(m_fSpGroup2SmallClass1ClearCutLog);
    mp_oChildBehaviors[1].AddRequiredData(m_fSpGroup2SmallClass2ClearCutLog);
    mp_oChildBehaviors[1].AddRequiredData(m_fSpGroup2SmallClass3ClearCutLog);
    mp_oChildBehaviors[1].AddRequiredData(m_fSpGroup2SmallClass4ClearCutLog);
    mp_oChildBehaviors[1].AddRequiredData(m_fSpGroup2SmallClass5ClearCutLog);
    mp_oChildBehaviors[1].AddRequiredData(m_fSpGroup3SmallClass1ClearCutLog);
    mp_oChildBehaviors[1].AddRequiredData(m_fSpGroup3SmallClass2ClearCutLog);
    mp_oChildBehaviors[1].AddRequiredData(m_fSpGroup3SmallClass3ClearCutLog);
    mp_oChildBehaviors[1].AddRequiredData(m_fSpGroup3SmallClass4ClearCutLog);
    mp_oChildBehaviors[1].AddRequiredData(m_fSpGroup3SmallClass5ClearCutLog);
    mp_oChildBehaviors[1].AddRequiredData(m_fSpGroup1LargeClass1ClearCutLog);
    mp_oChildBehaviors[1].AddRequiredData(m_fSpGroup1LargeClass2ClearCutLog);
    mp_oChildBehaviors[1].AddRequiredData(m_fSpGroup1LargeClass3ClearCutLog);
    mp_oChildBehaviors[1].AddRequiredData(m_fSpGroup1LargeClass4ClearCutLog);
    mp_oChildBehaviors[1].AddRequiredData(m_fSpGroup1LargeClass5ClearCutLog);
    mp_oChildBehaviors[1].AddRequiredData(m_fSpGroup2LargeClass1ClearCutLog);
    mp_oChildBehaviors[1].AddRequiredData(m_fSpGroup2LargeClass2ClearCutLog);
    mp_oChildBehaviors[1].AddRequiredData(m_fSpGroup2LargeClass3ClearCutLog);
    mp_oChildBehaviors[1].AddRequiredData(m_fSpGroup2LargeClass4ClearCutLog);
    mp_oChildBehaviors[1].AddRequiredData(m_fSpGroup2LargeClass5ClearCutLog);
    mp_oChildBehaviors[1].AddRequiredData(m_fSpGroup3LargeClass1ClearCutLog);
    mp_oChildBehaviors[1].AddRequiredData(m_fSpGroup3LargeClass2ClearCutLog);
    mp_oChildBehaviors[1].AddRequiredData(m_fSpGroup3LargeClass3ClearCutLog);
    mp_oChildBehaviors[1].AddRequiredData(m_fSpGroup3LargeClass4ClearCutLog);
    mp_oChildBehaviors[1].AddRequiredData(m_fSpGroup3LargeClass5ClearCutLog);
    mp_oChildBehaviors[1].AddRequiredData(m_fPartCutSmallLogMeanDiam);
    mp_oChildBehaviors[1].AddRequiredData(m_fPartCutLargeLogMeanDiam);
    mp_oChildBehaviors[1].AddRequiredData(m_fGapCutSmallLogMeanDiam);
    mp_oChildBehaviors[1].AddRequiredData(m_fGapCutLargeLogMeanDiam);
    mp_oChildBehaviors[1].AddRequiredData(m_fClearCutSmallLogMeanDiam);
    mp_oChildBehaviors[1].AddRequiredData(m_fClearCutLargeLogMeanDiam);

    mp_oAllData.add(m_fScarSoilA);
    mp_oAllData.add(m_fScarSoilB);
    mp_oAllData.add(m_fTipUpA);
    mp_oAllData.add(m_fTipUpB);
    mp_oAllData.add(m_fFreshlogA);
    mp_oAllData.add(m_fFreshlogB);
    mp_oAllData.add(m_fDecayedlogA);
    mp_oAllData.add(m_fDecayedlogB);
    mp_oAllData.add(m_iMaxDecayTime);
    mp_oAllData.add(m_fInitCondScarSoil);
    mp_oAllData.add(m_fInitCondTipup);
    mp_oAllData.add(m_fInitCondFreshLog);
    mp_oAllData.add(m_fInitCondDecLog);
    mp_oAllData.add(m_fPartialCutScarSoil);
    mp_oAllData.add(m_fPartialCutTipup);
    mp_oAllData.add(m_fPartialCutFreshLog);
    mp_oAllData.add(m_fPartialCutDecLog);
    mp_oAllData.add(m_fGapCutScarSoil);
    mp_oAllData.add(m_fGapCutTipup);
    mp_oAllData.add(m_fGapCutFreshLog);
    mp_oAllData.add(m_fGapCutDecLog);
    mp_oAllData.add(m_fClearCutScarSoil);
    mp_oAllData.add(m_fClearCutTipup);
    mp_oAllData.add(m_fClearCutFreshLog);
    mp_oAllData.add(m_fClearCutDecLog);
    mp_oAllData.add(mp_fProportionOfDeadThatFall);
    mp_oAllData.add(mp_fProportionOfFallenThatUproot);
    mp_oAllData.add(mp_fProportionOfSnagsThatUproot);
    mp_oAllData.add(m_fRootTipupFactor);
    mp_oAllData.add(m_fMossProportion);
    mp_oAllData.add(m_iDirectionalTreeFall);
    mp_oAllData.add(mp_iSpeciesGroup);
    mp_oAllData.add(m_fLogSizeClassBoundary);
    mp_oAllData.add(m_fPropFallenTreesDecayClass1);
    mp_oAllData.add(m_fPropFallenTreesDecayClass2);
    mp_oAllData.add(m_fPropFallenTreesDecayClass3);
    mp_oAllData.add(m_fPropFallenTreesDecayClass4);
    mp_oAllData.add(m_fPropFallenTreesDecayClass5);
    mp_oAllData.add(m_fPropFallenSnagsDecayClass1);
    mp_oAllData.add(m_fPropFallenSnagsDecayClass2);
    mp_oAllData.add(m_fPropFallenSnagsDecayClass3);
    mp_oAllData.add(m_fPropFallenSnagsDecayClass4);
    mp_oAllData.add(m_fPropFallenSnagsDecayClass5);
    mp_oAllData.add(m_fSpGroup1SmallClass1Alpha);
    mp_oAllData.add(m_fSpGroup1SmallClass2Alpha);
    mp_oAllData.add(m_fSpGroup1SmallClass3Alpha);
    mp_oAllData.add(m_fSpGroup1SmallClass4Alpha);
    mp_oAllData.add(m_fSpGroup1SmallClass5Alpha);
    mp_oAllData.add(m_fSpGroup2SmallClass1Alpha);
    mp_oAllData.add(m_fSpGroup2SmallClass2Alpha);
    mp_oAllData.add(m_fSpGroup2SmallClass3Alpha);
    mp_oAllData.add(m_fSpGroup2SmallClass4Alpha);
    mp_oAllData.add(m_fSpGroup2SmallClass5Alpha);
    mp_oAllData.add(m_fSpGroup3SmallClass1Alpha);
    mp_oAllData.add(m_fSpGroup3SmallClass2Alpha);
    mp_oAllData.add(m_fSpGroup3SmallClass3Alpha);
    mp_oAllData.add(m_fSpGroup3SmallClass4Alpha);
    mp_oAllData.add(m_fSpGroup3SmallClass5Alpha);
    mp_oAllData.add(m_fSpGroup1LargeClass1Alpha);
    mp_oAllData.add(m_fSpGroup1LargeClass2Alpha);
    mp_oAllData.add(m_fSpGroup1LargeClass3Alpha);
    mp_oAllData.add(m_fSpGroup1LargeClass4Alpha);
    mp_oAllData.add(m_fSpGroup1LargeClass5Alpha);
    mp_oAllData.add(m_fSpGroup2LargeClass1Alpha);
    mp_oAllData.add(m_fSpGroup2LargeClass2Alpha);
    mp_oAllData.add(m_fSpGroup2LargeClass3Alpha);
    mp_oAllData.add(m_fSpGroup2LargeClass4Alpha);
    mp_oAllData.add(m_fSpGroup2LargeClass5Alpha);
    mp_oAllData.add(m_fSpGroup3LargeClass1Alpha);
    mp_oAllData.add(m_fSpGroup3LargeClass2Alpha);
    mp_oAllData.add(m_fSpGroup3LargeClass3Alpha);
    mp_oAllData.add(m_fSpGroup3LargeClass4Alpha);
    mp_oAllData.add(m_fSpGroup3LargeClass5Alpha);
    mp_oAllData.add(m_fSpGroup1SmallClass1Beta);
    mp_oAllData.add(m_fSpGroup1SmallClass2Beta);
    mp_oAllData.add(m_fSpGroup1SmallClass3Beta);
    mp_oAllData.add(m_fSpGroup1SmallClass4Beta);
    mp_oAllData.add(m_fSpGroup1SmallClass5Beta);
    mp_oAllData.add(m_fSpGroup2SmallClass1Beta);
    mp_oAllData.add(m_fSpGroup2SmallClass2Beta);
    mp_oAllData.add(m_fSpGroup2SmallClass3Beta);
    mp_oAllData.add(m_fSpGroup2SmallClass4Beta);
    mp_oAllData.add(m_fSpGroup2SmallClass5Beta);
    mp_oAllData.add(m_fSpGroup3SmallClass1Beta);
    mp_oAllData.add(m_fSpGroup3SmallClass2Beta);
    mp_oAllData.add(m_fSpGroup3SmallClass3Beta);
    mp_oAllData.add(m_fSpGroup3SmallClass4Beta);
    mp_oAllData.add(m_fSpGroup3SmallClass5Beta);
    mp_oAllData.add(m_fSpGroup1LargeClass1Beta);
    mp_oAllData.add(m_fSpGroup1LargeClass2Beta);
    mp_oAllData.add(m_fSpGroup1LargeClass3Beta);
    mp_oAllData.add(m_fSpGroup1LargeClass4Beta);
    mp_oAllData.add(m_fSpGroup1LargeClass5Beta);
    mp_oAllData.add(m_fSpGroup2LargeClass1Beta);
    mp_oAllData.add(m_fSpGroup2LargeClass2Beta);
    mp_oAllData.add(m_fSpGroup2LargeClass3Beta);
    mp_oAllData.add(m_fSpGroup2LargeClass4Beta);
    mp_oAllData.add(m_fSpGroup2LargeClass5Beta);
    mp_oAllData.add(m_fSpGroup3LargeClass1Beta);
    mp_oAllData.add(m_fSpGroup3LargeClass2Beta);
    mp_oAllData.add(m_fSpGroup3LargeClass3Beta);
    mp_oAllData.add(m_fSpGroup3LargeClass4Beta);
    mp_oAllData.add(m_fSpGroup3LargeClass5Beta);
    mp_oAllData.add(m_fInitSmallLogMeanDiam);
    mp_oAllData.add(m_fInitLargeLogMeanDiam);
    mp_oAllData.add(m_fSpGroup1SmallClass1InitLog);
    mp_oAllData.add(m_fSpGroup1SmallClass2InitLog);
    mp_oAllData.add(m_fSpGroup1SmallClass3InitLog);
    mp_oAllData.add(m_fSpGroup1SmallClass4InitLog);
    mp_oAllData.add(m_fSpGroup1SmallClass5InitLog);
    mp_oAllData.add(m_fSpGroup2SmallClass1InitLog);
    mp_oAllData.add(m_fSpGroup2SmallClass2InitLog);
    mp_oAllData.add(m_fSpGroup2SmallClass3InitLog);
    mp_oAllData.add(m_fSpGroup2SmallClass4InitLog);
    mp_oAllData.add(m_fSpGroup2SmallClass5InitLog);
    mp_oAllData.add(m_fSpGroup3SmallClass1InitLog);
    mp_oAllData.add(m_fSpGroup3SmallClass2InitLog);
    mp_oAllData.add(m_fSpGroup3SmallClass3InitLog);
    mp_oAllData.add(m_fSpGroup3SmallClass4InitLog);
    mp_oAllData.add(m_fSpGroup3SmallClass5InitLog);
    mp_oAllData.add(m_fSpGroup1LargeClass1InitLog);
    mp_oAllData.add(m_fSpGroup1LargeClass2InitLog);
    mp_oAllData.add(m_fSpGroup1LargeClass3InitLog);
    mp_oAllData.add(m_fSpGroup1LargeClass4InitLog);
    mp_oAllData.add(m_fSpGroup1LargeClass5InitLog);
    mp_oAllData.add(m_fSpGroup2LargeClass1InitLog);
    mp_oAllData.add(m_fSpGroup2LargeClass2InitLog);
    mp_oAllData.add(m_fSpGroup2LargeClass3InitLog);
    mp_oAllData.add(m_fSpGroup2LargeClass4InitLog);
    mp_oAllData.add(m_fSpGroup2LargeClass5InitLog);
    mp_oAllData.add(m_fSpGroup3LargeClass1InitLog);
    mp_oAllData.add(m_fSpGroup3LargeClass2InitLog);
    mp_oAllData.add(m_fSpGroup3LargeClass3InitLog);
    mp_oAllData.add(m_fSpGroup3LargeClass4InitLog);
    mp_oAllData.add(m_fSpGroup3LargeClass5InitLog);
    mp_oAllData.add(m_fPartialCutScarSoil);
    mp_oAllData.add(m_fPartialCutTipup);
    mp_oAllData.add(m_fSpGroup1SmallClass1PartCutLog);
    mp_oAllData.add(m_fSpGroup1SmallClass2PartCutLog);
    mp_oAllData.add(m_fSpGroup1SmallClass3PartCutLog);
    mp_oAllData.add(m_fSpGroup1SmallClass4PartCutLog);
    mp_oAllData.add(m_fSpGroup1SmallClass5PartCutLog);
    mp_oAllData.add(m_fSpGroup2SmallClass1PartCutLog);
    mp_oAllData.add(m_fSpGroup2SmallClass2PartCutLog);
    mp_oAllData.add(m_fSpGroup2SmallClass3PartCutLog);
    mp_oAllData.add(m_fSpGroup2SmallClass4PartCutLog);
    mp_oAllData.add(m_fSpGroup2SmallClass5PartCutLog);
    mp_oAllData.add(m_fSpGroup3SmallClass1PartCutLog);
    mp_oAllData.add(m_fSpGroup3SmallClass2PartCutLog);
    mp_oAllData.add(m_fSpGroup3SmallClass3PartCutLog);
    mp_oAllData.add(m_fSpGroup3SmallClass4PartCutLog);
    mp_oAllData.add(m_fSpGroup3SmallClass5PartCutLog);
    mp_oAllData.add(m_fSpGroup1LargeClass1PartCutLog);
    mp_oAllData.add(m_fSpGroup1LargeClass2PartCutLog);
    mp_oAllData.add(m_fSpGroup1LargeClass3PartCutLog);
    mp_oAllData.add(m_fSpGroup1LargeClass4PartCutLog);
    mp_oAllData.add(m_fSpGroup1LargeClass5PartCutLog);
    mp_oAllData.add(m_fSpGroup2LargeClass1PartCutLog);
    mp_oAllData.add(m_fSpGroup2LargeClass2PartCutLog);
    mp_oAllData.add(m_fSpGroup2LargeClass3PartCutLog);
    mp_oAllData.add(m_fSpGroup2LargeClass4PartCutLog);
    mp_oAllData.add(m_fSpGroup2LargeClass5PartCutLog);
    mp_oAllData.add(m_fSpGroup3LargeClass1PartCutLog);
    mp_oAllData.add(m_fSpGroup3LargeClass2PartCutLog);
    mp_oAllData.add(m_fSpGroup3LargeClass3PartCutLog);
    mp_oAllData.add(m_fSpGroup3LargeClass4PartCutLog);
    mp_oAllData.add(m_fSpGroup3LargeClass5PartCutLog);
    mp_oAllData.add(m_fGapCutScarSoil);
    mp_oAllData.add(m_fGapCutTipup);
    mp_oAllData.add(m_fSpGroup1SmallClass1GapCutLog);
    mp_oAllData.add(m_fSpGroup1SmallClass2GapCutLog);
    mp_oAllData.add(m_fSpGroup1SmallClass3GapCutLog);
    mp_oAllData.add(m_fSpGroup1SmallClass4GapCutLog);
    mp_oAllData.add(m_fSpGroup1SmallClass5GapCutLog);
    mp_oAllData.add(m_fSpGroup2SmallClass1GapCutLog);
    mp_oAllData.add(m_fSpGroup2SmallClass2GapCutLog);
    mp_oAllData.add(m_fSpGroup2SmallClass3GapCutLog);
    mp_oAllData.add(m_fSpGroup2SmallClass4GapCutLog);
    mp_oAllData.add(m_fSpGroup2SmallClass5GapCutLog);
    mp_oAllData.add(m_fSpGroup3SmallClass1GapCutLog);
    mp_oAllData.add(m_fSpGroup3SmallClass2GapCutLog);
    mp_oAllData.add(m_fSpGroup3SmallClass3GapCutLog);
    mp_oAllData.add(m_fSpGroup3SmallClass4GapCutLog);
    mp_oAllData.add(m_fSpGroup3SmallClass5GapCutLog);
    mp_oAllData.add(m_fSpGroup1LargeClass1GapCutLog);
    mp_oAllData.add(m_fSpGroup1LargeClass2GapCutLog);
    mp_oAllData.add(m_fSpGroup1LargeClass3GapCutLog);
    mp_oAllData.add(m_fSpGroup1LargeClass4GapCutLog);
    mp_oAllData.add(m_fSpGroup1LargeClass5GapCutLog);
    mp_oAllData.add(m_fSpGroup2LargeClass1GapCutLog);
    mp_oAllData.add(m_fSpGroup2LargeClass2GapCutLog);
    mp_oAllData.add(m_fSpGroup2LargeClass3GapCutLog);
    mp_oAllData.add(m_fSpGroup2LargeClass4GapCutLog);
    mp_oAllData.add(m_fSpGroup2LargeClass5GapCutLog);
    mp_oAllData.add(m_fSpGroup3LargeClass1GapCutLog);
    mp_oAllData.add(m_fSpGroup3LargeClass2GapCutLog);
    mp_oAllData.add(m_fSpGroup3LargeClass3GapCutLog);
    mp_oAllData.add(m_fSpGroup3LargeClass4GapCutLog);
    mp_oAllData.add(m_fSpGroup3LargeClass5GapCutLog);
    mp_oAllData.add(m_fClearCutScarSoil);
    mp_oAllData.add(m_fClearCutTipup);
    mp_oAllData.add(m_fSpGroup1SmallClass1ClearCutLog);
    mp_oAllData.add(m_fSpGroup1SmallClass2ClearCutLog);
    mp_oAllData.add(m_fSpGroup1SmallClass3ClearCutLog);
    mp_oAllData.add(m_fSpGroup1SmallClass4ClearCutLog);
    mp_oAllData.add(m_fSpGroup1SmallClass5ClearCutLog);
    mp_oAllData.add(m_fSpGroup2SmallClass1ClearCutLog);
    mp_oAllData.add(m_fSpGroup2SmallClass2ClearCutLog);
    mp_oAllData.add(m_fSpGroup2SmallClass3ClearCutLog);
    mp_oAllData.add(m_fSpGroup2SmallClass4ClearCutLog);
    mp_oAllData.add(m_fSpGroup2SmallClass5ClearCutLog);
    mp_oAllData.add(m_fSpGroup3SmallClass1ClearCutLog);
    mp_oAllData.add(m_fSpGroup3SmallClass2ClearCutLog);
    mp_oAllData.add(m_fSpGroup3SmallClass3ClearCutLog);
    mp_oAllData.add(m_fSpGroup3SmallClass4ClearCutLog);
    mp_oAllData.add(m_fSpGroup3SmallClass5ClearCutLog);
    mp_oAllData.add(m_fSpGroup1LargeClass1ClearCutLog);
    mp_oAllData.add(m_fSpGroup1LargeClass2ClearCutLog);
    mp_oAllData.add(m_fSpGroup1LargeClass3ClearCutLog);
    mp_oAllData.add(m_fSpGroup1LargeClass4ClearCutLog);
    mp_oAllData.add(m_fSpGroup1LargeClass5ClearCutLog);
    mp_oAllData.add(m_fSpGroup2LargeClass1ClearCutLog);
    mp_oAllData.add(m_fSpGroup2LargeClass2ClearCutLog);
    mp_oAllData.add(m_fSpGroup2LargeClass3ClearCutLog);
    mp_oAllData.add(m_fSpGroup2LargeClass4ClearCutLog);
    mp_oAllData.add(m_fSpGroup2LargeClass5ClearCutLog);
    mp_oAllData.add(m_fSpGroup3LargeClass1ClearCutLog);
    mp_oAllData.add(m_fSpGroup3LargeClass2ClearCutLog);
    mp_oAllData.add(m_fSpGroup3LargeClass3ClearCutLog);
    mp_oAllData.add(m_fSpGroup3LargeClass4ClearCutLog);
    mp_oAllData.add(m_fSpGroup3LargeClass5ClearCutLog);
    mp_oAllData.add(m_fPartCutSmallLogMeanDiam);
    mp_oAllData.add(m_fPartCutLargeLogMeanDiam);
    mp_oAllData.add(m_fGapCutSmallLogMeanDiam);
    mp_oAllData.add(m_fGapCutLargeLogMeanDiam);
    mp_oAllData.add(m_fClearCutSmallLogMeanDiam);
    mp_oAllData.add(m_fClearCutLargeLogMeanDiam);
    
  }

  /**
   * Sets up the substrate grids.
   * @param oPop Tree population.
   * @throws ModelException
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   * <br>July 21, 2004:  Added calcs grids (LEM)
   */
  public void DoSetup(TreePopulation oPop) throws ModelException {

    Grid oNewGrid;
    int i, j, k, iCount;

    //************************
     // Regular substrate
     //************************
    String sGridName = "Substrate";

    //Create the six data members
    DataMember[] p_oDataMembers = new DataMember[6];

    p_oDataMembers[0] = new DataMember("Proportion of scarified soil",
                                       "scarsoil",
                                       DataMember.FLOAT);
    p_oDataMembers[1] = new DataMember("Proportion of tip-up mounds",
                                       "tipup", DataMember.FLOAT);
    p_oDataMembers[2] = new DataMember("Proportion of fresh logs", "freshlog",
                                       DataMember.FLOAT);
    p_oDataMembers[3] = new DataMember("Proportion of decayed logs", "declog",
                                       DataMember.FLOAT);
    p_oDataMembers[4] = new DataMember("Proportion of forest floor moss",
                                       "ffmoss",
                                       DataMember.FLOAT);
    p_oDataMembers[5] = new DataMember("Proportion of forest floor litter",
                                       "fflitter",
                                       DataMember.FLOAT);

    DataMember[] p_oPackageDataMembers = new DataMember[4];

    p_oPackageDataMembers[0] = new DataMember("Substrate cohort age", "age",
                                              DataMember.INTEGER);
    p_oPackageDataMembers[1] = new DataMember(
        "Substrate cohort new scarified soil substrate", "scarsoil",
        DataMember.FLOAT);
    p_oPackageDataMembers[2] = new DataMember(
        "Substrate cohort new tip-up mounds substrate",
        "tipup", DataMember.FLOAT);
    p_oPackageDataMembers[3] = new DataMember("Substrate cohort new fresh logs",
                                              "freshlog",
                                              DataMember.FLOAT);

    oNewGrid = new Grid(sGridName, p_oDataMembers, p_oPackageDataMembers, 8, 8);

    //Add to the substrate behavior
    oNewGrid = m_oManager.AddGrid(oNewGrid);
    GetBehaviorByKey("substrate").AddGrid(oNewGrid);

    //************************
    // Regular substrate calculations grid
    //************************
    sGridName = "substratecalcs";

    //Create the data members
    int iDecayTimesteps = (int) java.lang.Math.ceil( (float) m_iMaxDecayTime.
        GetValue() / m_oManager.GetPlot().GetNumberOfYearsPerTimestep());
    p_oDataMembers = new DataMember[1 + (2*iDecayTimesteps)];

    p_oDataMembers[0] = new DataMember("Amount of new tip-up mounds",
                                       "newtipup",
                                       DataMember.FLOAT);
    for (i = 0; i < iDecayTimesteps; i++) {
      p_oDataMembers[1 +
          i] = new DataMember("Fresh logs " + String.valueOf(i) +
                              " timesteps ago",
                              "freshlog_" + String.valueOf(i),
                              DataMember.FLOAT);
    }
    for (i = 0; i < iDecayTimesteps; i++) {
      p_oDataMembers[(1 + iDecayTimesteps) +
          i] = new DataMember("Decayed logs " + String.valueOf(i) +
                              " timesteps ago",
                              "declog_" + String.valueOf(i),
                              DataMember.FLOAT);
    }
    oNewGrid = new Grid(sGridName, p_oDataMembers, null, 8, 8);
    //Add to the substrate behavior
    oNewGrid = m_oManager.AddGrid(oNewGrid);
    GetBehaviorByKey("substrate").AddGrid(oNewGrid);
    
    //************************
    //Detailed substrate grid
    //************************
    sGridName = "DetailedSubstrate";

    //Create the six data members
    p_oDataMembers = new DataMember[66];

    p_oDataMembers[0] = new DataMember("Proportion of scarified soil",
                                       "scarsoil",
                                       DataMember.FLOAT);
    p_oDataMembers[1] = new DataMember("Proportion of tip-up mounds",
                                       "tipup", DataMember.FLOAT);
    p_oDataMembers[2] = new DataMember("Proportion of forest floor moss",
                                       "ffmoss",
                                       DataMember.FLOAT);
    p_oDataMembers[3] = new DataMember("Proportion of forest floor litter",
                                       "fflitter",
                                       DataMember.FLOAT);
    p_oDataMembers[4] = new DataMember("Total Log", "totallog", DataMember.FLOAT);
    p_oDataMembers[5] = new DataMember("Total Log Volume", "totallogvol", DataMember.FLOAT);
    iCount = 6;
    for (i = 1; i <= 5; i++) { //decay class
    	for (j = 1; j <= 3; j++) { //species group
    		p_oDataMembers[iCount] = new DataMember(
    				"Prop Sp Group " + j + " Small Decay " + i, 
    				"loggroup" + j + "smalldecay" + i, DataMember.FLOAT);     		
    		iCount++;
    		p_oDataMembers[iCount] = new DataMember(
    				"Prop Sp Group " + j + " Large Decay " + i, 
    				"loggroup" + j + "largedecay" + i, DataMember.FLOAT);     		
    		iCount++;
    		p_oDataMembers[iCount] = new DataMember(
    				"Vol Sp Group " + j + " Small Decay " + i, 
    				"vloggroup" + j + "smalldecay" + i, DataMember.FLOAT);     		
    		iCount++;
    		p_oDataMembers[iCount] = new DataMember(
    				"Vol Sp Group " + j + " Large Decay " + i, 
    				"vloggroup" + j + "largedecay" + i, DataMember.FLOAT);     		
    		iCount++;
    	}
    }

    p_oPackageDataMembers = new DataMember[3];
    p_oPackageDataMembers[0] = new DataMember("Substrate cohort age", "age",
                                              DataMember.INTEGER);
    p_oPackageDataMembers[1] = new DataMember(
        "Substrate cohort new scarified soil substrate", "scarsoil",
        DataMember.FLOAT);
    p_oPackageDataMembers[2] = new DataMember(
        "Substrate cohort new tip-up mounds substrate",
        "tipup", DataMember.FLOAT);

    oNewGrid = new Grid(sGridName, p_oDataMembers, p_oPackageDataMembers, 8, 8);

    //Add to the detailed substrate behavior
    oNewGrid = m_oManager.AddGrid(oNewGrid);
    GetBehaviorByKey("Detailed Substrate").AddGrid(oNewGrid);
    
    //************************
    // Detailed substrate calculations grid
    //************************
    sGridName = "detailedsubstratecalcs";

    //Create the data members
    p_oDataMembers = new DataMember[31 + (30*iDecayTimesteps)];

    p_oDataMembers[0] = new DataMember("Amount of new tip-up mounds",
                                       "newtipup",
                                       DataMember.FLOAT);

    iCount = 1;
    for (i = 1; i <= 5; i++) { //decay class
    	for (j = 1; j <= 3; j++) { //species group
    		p_oDataMembers[iCount] = new DataMember(
    				"New Log Area Sp Group " + j + " Small Decay " + i, 
    				"newloggroup" + j + "smalldecay" + i, DataMember.FLOAT);     		
    		iCount++;
    		p_oDataMembers[iCount] = new DataMember(
    				"New Log Area Sp Group " + j + " Large Decay " + i, 
    				"newloggroup" + j + "largedecay" + i, DataMember.FLOAT);     		
    		iCount++;
    		for (k = 0; k < iDecayTimesteps; k++) {
    		  p_oDataMembers[iCount] = new DataMember(
    		  	"Prop Sp Group " + j + " Small Decay " + i + " Timestep " + k, 
    				"loggroup" + j + "smalldecay" + i + "_" + k, DataMember.FLOAT);     		
    		  iCount++;
    		  p_oDataMembers[iCount] = new DataMember(
      		  	"Prop Sp Group " + j + " Large Decay " + i + " Timestep " + k, 
      				"loggroup" + j + "largedecay" + i + "_" + k, DataMember.FLOAT);     		
      		  iCount++;
    		}
    	}
    }
    oNewGrid = new Grid(sGridName, p_oDataMembers, null, 8, 8);
    //Add to the substrate behavior
    oNewGrid = m_oManager.AddGrid(oNewGrid);
    GetBehaviorByKey("Detailed Substrate").AddGrid(oNewGrid);

    //************************
     // Defaults
     //************************
      //Set the proportion of snags that uproot to all 0s, in case snags aren't
      //being used
    int iNumSpecies = oPop.GetNumberOfSpecies();
    if (mp_fProportionOfSnagsThatUproot.GetValue().size() == 0) {
      for (i = 0; i < iNumSpecies; i++) {
        mp_fProportionOfSnagsThatUproot.GetValue().add(new Float(0));
      }
    }
    
    //Detailed substrate setup
    //Set up the species group vector - values display in reverse order
    ModelEnum oVal;
		if (mp_iSpeciesGroup.GetValue().size() == 0) {
			for (i = 0; i < iNumSpecies; i++) {
				oVal = new ModelEnum(new int[] {3, 2, 1},	new String[] {"3", "2", "1"}, "", "");
				oVal.SetValue(1);
				mp_iSpeciesGroup.GetValue().add(oVal);
			}
		} else {
			// We may have added species - add enums if appropriate
			for (i = 0; i < iNumSpecies; i++) {
				if (null == mp_iSpeciesGroup.GetValue().elementAt(i)) {
					oVal = new ModelEnum(new int[] {3, 2, 1},	new String[] {"3", "2", "1"}, "", "");
					oVal.SetValue(1);
					mp_iSpeciesGroup.GetValue().remove(i);
					mp_iSpeciesGroup.GetValue().add(i, oVal);
				}
			}
		}
  }

  /**
   * Overridden from WorkerBase.  This is on the lookout for changes to
   * m_iMaxDecayTime, so DoSetup() can be triggered and the grids recreated.
   * @param oDataMember Data member to set.
   * @param oData Data value to set into data member.
   * @throws ModelException if data is of the wrong type.
    */
  protected void SetSingleValue(ModelData oDataMember, Object oData) throws
      ModelException {
    super.SetSingleValue(oDataMember, oData);
    if (oDataMember.equals(m_iMaxDecayTime)) {
      DoSetup(m_oManager.GetTreePopulation());
    }
  }

  /**
   * Overridden to provide backwards compatibility - I changed XML tag
   * names.  This was put in for 6.05 - remove after several more versions.
   * @param sXMLTag XML tag of data object whose value is to be set.
   * @param sXMLParentTag The immediate parent tag that sXMLTag is within.
   * @param oAttributes Attributes of the object.  Ignored, but may be needed
   * by overriding objects.
   * @param oData Data value, either String or type appropriate to the data
   * type
   * @return true if the value was set successfully; false if the value could
   * not be found.
   * @throws ModelException if the value could not be assigned to the data
   * object.
   */
  public boolean SetSingleValueByXMLTag(String sXMLTag, String sXMLParentTag,
                                        org.xml.sax.Attributes oAttributes,
                                        Object oData) throws
      ModelException {

    if (sXMLParentTag.equals(m_sXMLTag)) {
      if (sXMLTag.equals("su_DecayedLogToForestFloor") ||
          sXMLTag.equals("su_mossyLogToCanMoss")) {
        //Old straight proportion of decayed log decay - transform it to
        //a and b
        Float oFloat;
        if (oData instanceof String) {
          oFloat = new Float( (String) oData);
        }
        else {
          oFloat = (Float) oData;
        }
        m_fDecayedlogA.SetValue((float)java.lang.Math.log((1 - oFloat.floatValue())));
        m_fDecayedlogB.SetValue(1);
        return true;
      }
      else if (sXMLTag.equals("su_gapmossDecayAlpha")) {
        sXMLTag = "su_scarSoilDecayAlpha";
      }
      else if (sXMLTag.equals("su_gapmossDecayBeta")) {
        sXMLTag = "su_scarSoilDecayBeta";
      }
      else if (sXMLTag.equals("su_organicDecayAlpha")) {
        sXMLTag = "su_tipupDecayAlpha";
      }
      else if (sXMLTag.equals("su_organicDecayBeta")) {
        sXMLTag = "su_tipupDecayBeta";
      }
      else if (sXMLTag.equals("su_initialGapMoss")) {
        sXMLTag = "su_initialScarSoil";
      }
      else if (sXMLTag.equals("su_initialOrganic")) {
        sXMLTag = "su_initialTipup";
      }
      else if (sXMLTag.equals("su_initialMossyLog")) {
        sXMLTag = "su_initialDecayedLog";
      }
      else if (sXMLTag.equals("su_partialCutGapMoss")) {
        sXMLTag = "su_partialCutScarSoil";
      }
      else if (sXMLTag.equals("su_partialCutOrganic")) {
        sXMLTag = "su_partialCutTipup";
      }
      else if (sXMLTag.equals("su_partialCutMossyLog")) {
        sXMLTag = "su_partialCutDecayedLog";
      }
      else if (sXMLTag.equals("su_gapCutGapMoss")) {
        sXMLTag = "su_gapCutScarSoil";
      }
      else if (sXMLTag.equals("su_gapCutOrganic")) {
        sXMLTag = "su_gapCutTipup";
      }
      else if (sXMLTag.equals("su_gapCutMossyLog")) {
        sXMLTag = "su_gapCutDecayedLog";
      }
      else if (sXMLTag.equals("su_clearCutGapMoss")) {
        sXMLTag = "su_clearCutScarSoil";
      }
      else if (sXMLTag.equals("su_clearCutOrganic")) {
        sXMLTag = "su_clearCutTipup";
      }
      else if (sXMLTag.equals("su_clearCutMossyLog")) {
        sXMLTag = "su_clearCutDecayedLog";
      }
    }
    //Grid values
    /*else if (sXMLTag.equals("ma_floatCode")) {
      if (m_iGridParseIndex > -1) {
        String sLabel = oAttributes.getValue("label");
        boolean bUpdate = false;
        if (sLabel.equals("gapmoss")) {
          sLabel = "scarsoil";
          bUpdate = true;
        }
        else if (sLabel.equals("canmoss")) {
          sLabel = "ffmoss";
          bUpdate = true;
        }
        else if (sLabel.equals("organic")) {
          sLabel = "tipup";
          bUpdate = true;
        }
        else if (sLabel.equals("mossylog")) {
          sLabel = "declog";
          bUpdate = true;
        }
        //We can't set the value in oAttributes and call super, because
        //oAttributes won't let us; so we have to copy the setting code
        //here
        if (true == bUpdate) {
          int iIndex = new Integer( (String) oData).intValue();
          if (m_iPackageIndex > -1) {
            mp_oGrids[m_iGridParseIndex].SetGridPackageFloatCode(sLabel,
                iIndex);
            return true;
          }
          else {
            mp_oGrids[m_iGridParseIndex].SetGridFloatCode(sLabel, iIndex);
            return true;
          }
        }
      }
    } */
    return super.SetSingleValueByXMLTag(sXMLTag, sXMLParentTag, oAttributes,
                                        oData);
  }

  /**
   * Validates the data in preparation for parameter file writing or some such.
   * @throws ModelException if:
   * <ul>
   * <li>m_fRootTipupFactor or m_iMaxDecayTime are not positive numbers</li>
   * <li>if the values in mp_fProportionOfDeadThatFall,
   * mp_fProportionOfFallenThatUproot, and mp_fProportionOfSnagsThatUproot are
   * not proportions</li>
   * <li>m_fMossProportion isn't a proportion</li>
   * <li>any initial or harvest conditions value isn't a proportion</li>
   * <li>the total of any set of initial or harvest conditions is greater than
   * 1</li>
   * <li>Detailed substrate is enabled and:
   * <li>Detailed substrate betas are too large</li>
   * <li>Detailed substrate alphas are positive</li>
   * <li>Initial conditions proportions aren't less than 1</li>
   * <li>Partial cut conditions proportions aren't less than 1</li>
   * <li>Gap cut conditions proportions aren't less than 1</li>
   * <li>Clear cut conditions proportions aren't less than 1</li>
   * <li>Live tree decay class proportions don't add up to 1</li>
   * <li>Snag decay class proportions don't add up to 1</li>
   * <li>Mean small and large log diameters don't straddle the log size class
   * boundary</li>
   * <li>Snag dynamics behavior is not enabled</li>
   * </li>
   * </ul>
   * @param oPop Not used.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   * <br>April 22, 2005:  Added proportion checking (LEM)
   */
  public void ValidateData(TreePopulation oPop) throws ModelException {

    if (GetBehaviorByKey("substrate").m_bIsEnabled) {
      boolean[] p_bApplies = GetBehaviorByKey("substrate").GetWhichSpeciesUsed(
          oPop);

      MakeSureGreaterThan(m_fRootTipupFactor, 0);
      MakeSureGreaterThan(m_iMaxDecayTime, 0);
      MakeSureIsProportion(m_fMossProportion);
      MakeSureAllAreProportions(mp_fProportionOfDeadThatFall, p_bApplies);
      MakeSureAllAreProportions(mp_fProportionOfFallenThatUproot, p_bApplies);
      MakeSureAllAreProportions(mp_fProportionOfSnagsThatUproot, p_bApplies);
      MakeSureIsProportion(m_fInitCondDecLog);
      MakeSureIsProportion(m_fInitCondFreshLog);
      MakeSureIsProportion(m_fInitCondScarSoil);
      MakeSureIsProportion(m_fInitCondTipup);
      MakeSureIsProportion(m_fPartialCutDecLog);
      MakeSureIsProportion(m_fPartialCutFreshLog);
      MakeSureIsProportion(m_fPartialCutScarSoil);
      MakeSureIsProportion(m_fPartialCutTipup);
      MakeSureIsProportion(m_fGapCutDecLog);
      MakeSureIsProportion(m_fGapCutFreshLog);
      MakeSureIsProportion(m_fGapCutScarSoil);
      MakeSureIsProportion(m_fGapCutTipup);
      MakeSureIsProportion(m_fClearCutDecLog);
      MakeSureIsProportion(m_fClearCutFreshLog);
      MakeSureIsProportion(m_fClearCutScarSoil);
      MakeSureIsProportion(m_fClearCutTipup);

      if (m_fInitCondDecLog.GetValue() +
          m_fInitCondFreshLog.GetValue() +
          m_fInitCondScarSoil.GetValue() +
          m_fInitCondTipup.GetValue() > 1) {
        throw new ModelException(ErrorGUI.BAD_DATA, "JAVA",
            "The initial conditions values must not add up to be greater than 1.");
      }

      if (m_fPartialCutDecLog.GetValue() +
          m_fPartialCutFreshLog.GetValue() +
          m_fPartialCutScarSoil.GetValue() +
          m_fPartialCutTipup.GetValue() > 1) {
        throw new ModelException(ErrorGUI.BAD_DATA, "JAVA",
            "The partial cut values must not add up to be greater than 1.");
      }

      if (m_fGapCutDecLog.GetValue() +
          m_fGapCutFreshLog.GetValue() +
          m_fGapCutScarSoil.GetValue() +
          m_fGapCutTipup.GetValue() > 1) {
        throw new ModelException(ErrorGUI.BAD_DATA, "JAVA",
            "The gap cut values must not add up to be greater than 1.");
      }

      if (m_fClearCutDecLog.GetValue() +
          m_fClearCutFreshLog.GetValue() +
          m_fClearCutScarSoil.GetValue() +
          m_fClearCutTipup.GetValue() > 1) {
        throw new ModelException(ErrorGUI.BAD_DATA, "JAVA",
            "The clear cut values must not add up to be greater than 1.");
      }
    }
    
    if (GetBehaviorByKey("Detailed Substrate").m_bIsEnabled) {
      boolean[] p_bApplies = GetBehaviorByKey("Detailed Substrate").GetWhichSpeciesUsed(
          oPop);
      
      MakeSureAllAreProportions(mp_fProportionOfFallenThatUproot, p_bApplies);
      MakeSureAllAreProportions(mp_fProportionOfSnagsThatUproot, p_bApplies);
      MakeSureGreaterThan(m_fRootTipupFactor, 0);
      MakeSureGreaterThan(m_iMaxDecayTime, 0);
      MakeSureIsProportion(m_fMossProportion);
      MakeSureIsProportion(m_fInitCondScarSoil);
      MakeSureIsProportion(m_fInitCondTipup);
      MakeSureIsProportion(m_fPartialCutScarSoil);
      MakeSureIsProportion(m_fPartialCutTipup);
      MakeSureIsProportion(m_fGapCutScarSoil);
      MakeSureIsProportion(m_fGapCutTipup);
      MakeSureIsProportion(m_fClearCutScarSoil);
      MakeSureIsProportion(m_fClearCutTipup);
      
      //Check to see species groups
      ModelEnum oEnum;
      boolean bAnyGroup1 = false, bAnyGroup2 = false, bAnyGroup3 = false;
      for (int i = 0; i < mp_iSpeciesGroup.GetValue().size(); i++) {
      	oEnum = (ModelEnum) mp_iSpeciesGroup.GetValue().elementAt(i);
      	if (oEnum.GetValue() == 1) bAnyGroup1 = true;
      	else if (oEnum.GetValue() == 2) bAnyGroup2 = true;
      	else if (oEnum.GetValue() == 3) bAnyGroup3 = true;
      }
      
      //Check betas to make sure they are not too large and that the 
      //alpha-beta pair won't be too large to raise e to the power of it
      float fMaxBeta = (float)(30.0 / java.lang.Math.log(m_iMaxDecayTime.GetValue()));
      MakeSureLessThan(m_fScarSoilB, fMaxBeta);
      MakeSureLessThan(m_fTipUpB, fMaxBeta);
      
      if (m_fScarSoilA.GetValue() * 
          java.lang.Math.pow(m_iMaxDecayTime.GetValue(),m_fScarSoilB.GetValue()) < -30.0) {
        throw new ModelException(ErrorGUI.BAD_DATA, "JAVA",
          "Alpha * (beta ^ max timesteps) < -30 must be true for the " +
          "following parameters: " + m_fScarSoilA.GetDescriptor() + ", " + 
          m_fScarSoilB.GetDescriptor());
      }
      
      if (m_fTipUpA.GetValue() * 
          java.lang.Math.pow(m_iMaxDecayTime.GetValue(),m_fTipUpB.GetValue()) < -30.0) {
        throw new ModelException(ErrorGUI.BAD_DATA, "JAVA",
          "Alpha * (beta ^ max timesteps) < -30 must be true for the " +
          "following parameters: " + m_fTipUpA.GetDescriptor() + ", " + 
          m_fTipUpB.GetDescriptor());
      }
      if (bAnyGroup1) {
        MakeSureLessThan(m_fSpGroup1SmallClass1Beta, fMaxBeta);
        MakeSureLessThan(m_fSpGroup1SmallClass2Beta, fMaxBeta);
        MakeSureLessThan(m_fSpGroup1SmallClass3Beta, fMaxBeta);
        MakeSureLessThan(m_fSpGroup1SmallClass4Beta, fMaxBeta);
        MakeSureLessThan(m_fSpGroup1SmallClass5Beta, fMaxBeta);
        MakeSureLessThan(m_fSpGroup1LargeClass1Beta, fMaxBeta);
        MakeSureLessThan(m_fSpGroup1LargeClass2Beta, fMaxBeta);
        MakeSureLessThan(m_fSpGroup1LargeClass3Beta, fMaxBeta);
        MakeSureLessThan(m_fSpGroup1LargeClass4Beta, fMaxBeta);
        MakeSureLessThan(m_fSpGroup1LargeClass5Beta, fMaxBeta);
        
        if (m_fSpGroup1SmallClass1Alpha.GetValue() * 
            java.lang.Math.pow(m_iMaxDecayTime.GetValue(),
                m_fSpGroup1SmallClass1Beta.GetValue()) < -30.0) {
          throw new ModelException(ErrorGUI.BAD_DATA, "JAVA",
            "Alpha * (beta ^ max timesteps) < -30 must be true for the " +
            "following parameters: " + 
            m_fSpGroup1SmallClass1Alpha.GetDescriptor() + ", " + 
            m_fSpGroup1SmallClass1Beta.GetDescriptor());
        }
        
        if (m_fSpGroup1SmallClass2Alpha.GetValue() * 
            java.lang.Math.pow(m_iMaxDecayTime.GetValue(),
                m_fSpGroup1SmallClass2Beta.GetValue()) < -30.0) {
          throw new ModelException(ErrorGUI.BAD_DATA, "JAVA",
            "Alpha * (beta ^ max timesteps) < -30 must be true for the " +
            "following parameters: " + 
            m_fSpGroup1SmallClass2Alpha.GetDescriptor() + ", " + 
            m_fSpGroup1SmallClass2Beta.GetDescriptor());
        }
        
        if (m_fSpGroup1SmallClass3Alpha.GetValue() * 
            java.lang.Math.pow(m_iMaxDecayTime.GetValue(),
                m_fSpGroup1SmallClass3Beta.GetValue()) < -30.0) {
          throw new ModelException(ErrorGUI.BAD_DATA, "JAVA",
            "Alpha * (beta ^ max timesteps) < -30 must be true for the " +
            "following parameters: " + 
            m_fSpGroup1SmallClass3Alpha.GetDescriptor() + ", " + 
            m_fSpGroup1SmallClass3Beta.GetDescriptor());
        }
        
        if (m_fSpGroup1SmallClass4Alpha.GetValue() * 
            java.lang.Math.pow(m_iMaxDecayTime.GetValue(),
                m_fSpGroup1SmallClass4Beta.GetValue()) < -30.0) {
          throw new ModelException(ErrorGUI.BAD_DATA, "JAVA",
            "Alpha * (beta ^ max timesteps) < -30 must be true for the " +
            "following parameters: " + 
            m_fSpGroup1SmallClass4Alpha.GetDescriptor() + ", " + 
            m_fSpGroup1SmallClass4Beta.GetDescriptor());
        }
        
        if (m_fSpGroup1SmallClass5Alpha.GetValue() * 
            java.lang.Math.pow(m_iMaxDecayTime.GetValue(),
                m_fSpGroup1SmallClass5Beta.GetValue()) < -30.0) {
          throw new ModelException(ErrorGUI.BAD_DATA, "JAVA",
            "Alpha * (beta ^ max timesteps) < -30 must be true for the " +
            "following parameters: " + 
            m_fSpGroup1SmallClass5Alpha.GetDescriptor() + ", " + 
            m_fSpGroup1SmallClass5Beta.GetDescriptor());
        }
        
        if (m_fSpGroup1LargeClass1Alpha.GetValue() * 
            java.lang.Math.pow(m_iMaxDecayTime.GetValue(),
                m_fSpGroup1LargeClass1Beta.GetValue()) < -30.0) {
          throw new ModelException(ErrorGUI.BAD_DATA, "JAVA",
            "Alpha * (beta ^ max timesteps) < -30 must be true for the " +
            "following parameters: " + 
            m_fSpGroup1LargeClass1Alpha.GetDescriptor() + ", " + 
            m_fSpGroup1LargeClass1Beta.GetDescriptor());
        }
        
        if (m_fSpGroup1LargeClass2Alpha.GetValue() * 
            java.lang.Math.pow(m_iMaxDecayTime.GetValue(),
                m_fSpGroup1LargeClass2Beta.GetValue()) < -30.0) {
          throw new ModelException(ErrorGUI.BAD_DATA, "JAVA",
            "Alpha * (beta ^ max timesteps) < -30 must be true for the " +
            "following parameters: " + 
            m_fSpGroup1LargeClass2Alpha.GetDescriptor() + ", " + 
            m_fSpGroup1LargeClass2Beta.GetDescriptor());
        }
        
        if (m_fSpGroup1LargeClass3Alpha.GetValue() * 
            java.lang.Math.pow(m_iMaxDecayTime.GetValue(),
                m_fSpGroup1LargeClass3Beta.GetValue()) < -30.0) {
          throw new ModelException(ErrorGUI.BAD_DATA, "JAVA",
            "Alpha * (beta ^ max timesteps) < -30 must be true for the " +
            "following parameters: " + 
            m_fSpGroup1LargeClass3Alpha.GetDescriptor() + ", " + 
            m_fSpGroup1LargeClass3Beta.GetDescriptor());
        }
        
        if (m_fSpGroup1LargeClass4Alpha.GetValue() * 
            java.lang.Math.pow(m_iMaxDecayTime.GetValue(),
                m_fSpGroup1LargeClass4Beta.GetValue()) < -30.0) {
          throw new ModelException(ErrorGUI.BAD_DATA, "JAVA",
            "Alpha * (beta ^ max timesteps) < -30 must be true for the " +
            "following parameters: " + 
            m_fSpGroup1LargeClass4Alpha.GetDescriptor() + ", " + 
            m_fSpGroup1LargeClass4Beta.GetDescriptor());
        }
        
        if (m_fSpGroup1LargeClass5Alpha.GetValue() * 
            java.lang.Math.pow(m_iMaxDecayTime.GetValue(),
                m_fSpGroup1LargeClass5Beta.GetValue()) < -30.0) {
          throw new ModelException(ErrorGUI.BAD_DATA, "JAVA",
            "Alpha * (beta ^ max timesteps) < -30 must be true for the " +
            "following parameters: " + 
            m_fSpGroup1LargeClass5Alpha.GetDescriptor() + ", " + 
            m_fSpGroup1LargeClass5Beta.GetDescriptor());
        }
      }
      if (bAnyGroup2) {
        MakeSureLessThan(m_fSpGroup2SmallClass1Beta, fMaxBeta);
        MakeSureLessThan(m_fSpGroup2SmallClass2Beta, fMaxBeta);
        MakeSureLessThan(m_fSpGroup2SmallClass3Beta, fMaxBeta);
        MakeSureLessThan(m_fSpGroup2SmallClass4Beta, fMaxBeta);
        MakeSureLessThan(m_fSpGroup2SmallClass5Beta, fMaxBeta);
        MakeSureLessThan(m_fSpGroup2LargeClass1Beta, fMaxBeta);
        MakeSureLessThan(m_fSpGroup2LargeClass2Beta, fMaxBeta);
        MakeSureLessThan(m_fSpGroup2LargeClass3Beta, fMaxBeta);
        MakeSureLessThan(m_fSpGroup2LargeClass4Beta, fMaxBeta);
        MakeSureLessThan(m_fSpGroup2LargeClass5Beta, fMaxBeta);
        
        if (m_fSpGroup2SmallClass1Alpha.GetValue() * 
            java.lang.Math.pow(m_iMaxDecayTime.GetValue(),
                m_fSpGroup2SmallClass1Beta.GetValue()) < -30.0) {
          throw new ModelException(ErrorGUI.BAD_DATA, "JAVA",
            "Alpha * (beta ^ max timesteps) < -30 must be true for the " +
            "following parameters: " + 
            m_fSpGroup2SmallClass1Alpha.GetDescriptor() + ", " + 
            m_fSpGroup2SmallClass1Beta.GetDescriptor());
        }
        
        if (m_fSpGroup2SmallClass2Alpha.GetValue() * 
            java.lang.Math.pow(m_iMaxDecayTime.GetValue(),
                m_fSpGroup2SmallClass2Beta.GetValue()) < -30.0) {
          throw new ModelException(ErrorGUI.BAD_DATA, "JAVA",
            "Alpha * (beta ^ max timesteps) < -30 must be true for the " +
            "following parameters: " + 
            m_fSpGroup2SmallClass2Alpha.GetDescriptor() + ", " + 
            m_fSpGroup2SmallClass2Beta.GetDescriptor());
        }
        
        if (m_fSpGroup2SmallClass3Alpha.GetValue() * 
            java.lang.Math.pow(m_iMaxDecayTime.GetValue(),
                m_fSpGroup2SmallClass3Beta.GetValue()) < -30.0) {
          throw new ModelException(ErrorGUI.BAD_DATA, "JAVA",
            "Alpha * (beta ^ max timesteps) < -30 must be true for the " +
            "following parameters: " + 
            m_fSpGroup2SmallClass3Alpha.GetDescriptor() + ", " + 
            m_fSpGroup2SmallClass3Beta.GetDescriptor());
        }
        
        if (m_fSpGroup2SmallClass4Alpha.GetValue() * 
            java.lang.Math.pow(m_iMaxDecayTime.GetValue(),
                m_fSpGroup2SmallClass4Beta.GetValue()) < -30.0) {
          throw new ModelException(ErrorGUI.BAD_DATA, "JAVA",
            "Alpha * (beta ^ max timesteps) < -30 must be true for the " +
            "following parameters: " + 
            m_fSpGroup2SmallClass4Alpha.GetDescriptor() + ", " + 
            m_fSpGroup2SmallClass4Beta.GetDescriptor());
        }
        
        if (m_fSpGroup2SmallClass5Alpha.GetValue() * 
            java.lang.Math.pow(m_iMaxDecayTime.GetValue(),
                m_fSpGroup2SmallClass5Beta.GetValue()) < -30.0) {
          throw new ModelException(ErrorGUI.BAD_DATA, "JAVA",
            "Alpha * (beta ^ max timesteps) < -30 must be true for the " +
            "following parameters: " + 
            m_fSpGroup2SmallClass5Alpha.GetDescriptor() + ", " + 
            m_fSpGroup2SmallClass5Beta.GetDescriptor());
        }
        
        if (m_fSpGroup2LargeClass1Alpha.GetValue() * 
            java.lang.Math.pow(m_iMaxDecayTime.GetValue(),
                m_fSpGroup2LargeClass1Beta.GetValue()) < -30.0) {
          throw new ModelException(ErrorGUI.BAD_DATA, "JAVA",
            "Alpha * (beta ^ max timesteps) < -30 must be true for the " +
            "following parameters: " + 
            m_fSpGroup2LargeClass1Alpha.GetDescriptor() + ", " + 
            m_fSpGroup2LargeClass1Beta.GetDescriptor());
        }
        
        if (m_fSpGroup2LargeClass2Alpha.GetValue() * 
            java.lang.Math.pow(m_iMaxDecayTime.GetValue(),
                m_fSpGroup2LargeClass2Beta.GetValue()) < -30.0) {
          throw new ModelException(ErrorGUI.BAD_DATA, "JAVA",
            "Alpha * (beta ^ max timesteps) < -30 must be true for the " +
            "following parameters: " + 
            m_fSpGroup2LargeClass2Alpha.GetDescriptor() + ", " + 
            m_fSpGroup2LargeClass2Beta.GetDescriptor());
        }
        
        if (m_fSpGroup2LargeClass3Alpha.GetValue() * 
            java.lang.Math.pow(m_iMaxDecayTime.GetValue(),
                m_fSpGroup2LargeClass3Beta.GetValue()) < -30.0) {
          throw new ModelException(ErrorGUI.BAD_DATA, "JAVA",
            "Alpha * (beta ^ max timesteps) < -30 must be true for the " +
            "following parameters: " + 
            m_fSpGroup2LargeClass3Alpha.GetDescriptor() + ", " + 
            m_fSpGroup2LargeClass3Beta.GetDescriptor());
        }
        
        if (m_fSpGroup2LargeClass4Alpha.GetValue() * 
            java.lang.Math.pow(m_iMaxDecayTime.GetValue(),
                m_fSpGroup2LargeClass4Beta.GetValue()) < -30.0) {
          throw new ModelException(ErrorGUI.BAD_DATA, "JAVA",
            "Alpha * (beta ^ max timesteps) < -30 must be true for the " +
            "following parameters: " + 
            m_fSpGroup2LargeClass4Alpha.GetDescriptor() + ", " + 
            m_fSpGroup2LargeClass4Beta.GetDescriptor());
        }
        
        if (m_fSpGroup2LargeClass5Alpha.GetValue() * 
            java.lang.Math.pow(m_iMaxDecayTime.GetValue(),
                m_fSpGroup2LargeClass5Beta.GetValue()) < -30.0) {
          throw new ModelException(ErrorGUI.BAD_DATA, "JAVA",
            "Alpha * (beta ^ max timesteps) < -30 must be true for the " +
            "following parameters: " + 
            m_fSpGroup2LargeClass5Alpha.GetDescriptor() + ", " + 
            m_fSpGroup2LargeClass5Beta.GetDescriptor());
        }
      }
      if (bAnyGroup3) {
        MakeSureLessThan(m_fSpGroup3SmallClass1Beta, fMaxBeta);
        MakeSureLessThan(m_fSpGroup3SmallClass2Beta, fMaxBeta);
        MakeSureLessThan(m_fSpGroup3SmallClass3Beta, fMaxBeta);
        MakeSureLessThan(m_fSpGroup3SmallClass4Beta, fMaxBeta);
        MakeSureLessThan(m_fSpGroup3SmallClass5Beta, fMaxBeta);
        MakeSureLessThan(m_fSpGroup3LargeClass1Beta, fMaxBeta);
        MakeSureLessThan(m_fSpGroup3LargeClass2Beta, fMaxBeta);
        MakeSureLessThan(m_fSpGroup3LargeClass3Beta, fMaxBeta);
        MakeSureLessThan(m_fSpGroup3LargeClass4Beta, fMaxBeta);
        MakeSureLessThan(m_fSpGroup3LargeClass5Beta, fMaxBeta);
        
        if (m_fSpGroup3SmallClass1Alpha.GetValue() * 
            java.lang.Math.pow(m_iMaxDecayTime.GetValue(),
                m_fSpGroup3SmallClass1Beta.GetValue()) < -30.0) {
          throw new ModelException(ErrorGUI.BAD_DATA, "JAVA",
            "Alpha * (beta ^ max timesteps) < -30 must be true for the " +
            "following parameters: " + 
            m_fSpGroup3SmallClass1Alpha.GetDescriptor() + ", " + 
            m_fSpGroup3SmallClass1Beta.GetDescriptor());
        }
        
        if (m_fSpGroup3SmallClass2Alpha.GetValue() * 
            java.lang.Math.pow(m_iMaxDecayTime.GetValue(),
                m_fSpGroup3SmallClass2Beta.GetValue()) < -30.0) {
          throw new ModelException(ErrorGUI.BAD_DATA, "JAVA",
            "Alpha * (beta ^ max timesteps) < -30 must be true for the " +
            "following parameters: " + 
            m_fSpGroup3SmallClass2Alpha.GetDescriptor() + ", " + 
            m_fSpGroup3SmallClass2Beta.GetDescriptor());
        }
        
        if (m_fSpGroup3SmallClass3Alpha.GetValue() * 
            java.lang.Math.pow(m_iMaxDecayTime.GetValue(),
                m_fSpGroup3SmallClass3Beta.GetValue()) < -30.0) {
          throw new ModelException(ErrorGUI.BAD_DATA, "JAVA",
            "Alpha * (beta ^ max timesteps) < -30 must be true for the " +
            "following parameters: " + 
            m_fSpGroup3SmallClass3Alpha.GetDescriptor() + ", " + 
            m_fSpGroup3SmallClass3Beta.GetDescriptor());
        }
        
        if (m_fSpGroup3SmallClass4Alpha.GetValue() * 
            java.lang.Math.pow(m_iMaxDecayTime.GetValue(),
                m_fSpGroup3SmallClass4Beta.GetValue()) < -30.0) {
          throw new ModelException(ErrorGUI.BAD_DATA, "JAVA",
            "Alpha * (beta ^ max timesteps) < -30 must be true for the " +
            "following parameters: " + 
            m_fSpGroup3SmallClass4Alpha.GetDescriptor() + ", " + 
            m_fSpGroup3SmallClass4Beta.GetDescriptor());
        }
        
        if (m_fSpGroup3SmallClass5Alpha.GetValue() * 
            java.lang.Math.pow(m_iMaxDecayTime.GetValue(),
                m_fSpGroup3SmallClass5Beta.GetValue()) < -30.0) {
          throw new ModelException(ErrorGUI.BAD_DATA, "JAVA",
            "Alpha * (beta ^ max timesteps) < -30 must be true for the " +
            "following parameters: " + 
            m_fSpGroup3SmallClass5Alpha.GetDescriptor() + ", " + 
            m_fSpGroup3SmallClass5Beta.GetDescriptor());
        }
        
        if (m_fSpGroup3LargeClass1Alpha.GetValue() * 
            java.lang.Math.pow(m_iMaxDecayTime.GetValue(),
                m_fSpGroup3LargeClass1Beta.GetValue()) < -30.0) {
          throw new ModelException(ErrorGUI.BAD_DATA, "JAVA",
            "Alpha * (beta ^ max timesteps) < -30 must be true for the " +
            "following parameters: " + 
            m_fSpGroup3LargeClass1Alpha.GetDescriptor() + ", " + 
            m_fSpGroup3LargeClass1Beta.GetDescriptor());
        }
        
        if (m_fSpGroup3LargeClass2Alpha.GetValue() * 
            java.lang.Math.pow(m_iMaxDecayTime.GetValue(),
                m_fSpGroup3LargeClass2Beta.GetValue()) < -30.0) {
          throw new ModelException(ErrorGUI.BAD_DATA, "JAVA",
            "Alpha * (beta ^ max timesteps) < -30 must be true for the " +
            "following parameters: " + 
            m_fSpGroup3LargeClass2Alpha.GetDescriptor() + ", " + 
            m_fSpGroup3LargeClass2Beta.GetDescriptor());
        }
        
        if (m_fSpGroup3LargeClass3Alpha.GetValue() * 
            java.lang.Math.pow(m_iMaxDecayTime.GetValue(),
                m_fSpGroup3LargeClass3Beta.GetValue()) < -30.0) {
          throw new ModelException(ErrorGUI.BAD_DATA, "JAVA",
            "Alpha * (beta ^ max timesteps) < -30 must be true for the " +
            "following parameters: " + 
            m_fSpGroup3LargeClass3Alpha.GetDescriptor() + ", " + 
            m_fSpGroup3LargeClass3Beta.GetDescriptor());
        }
        
        if (m_fSpGroup3LargeClass4Alpha.GetValue() * 
            java.lang.Math.pow(m_iMaxDecayTime.GetValue(),
                m_fSpGroup3LargeClass4Beta.GetValue()) < -30.0) {
          throw new ModelException(ErrorGUI.BAD_DATA, "JAVA",
            "Alpha * (beta ^ max timesteps) < -30 must be true for the " +
            "following parameters: " + 
            m_fSpGroup3LargeClass4Alpha.GetDescriptor() + ", " + 
            m_fSpGroup3LargeClass4Beta.GetDescriptor());
        }
        
        if (m_fSpGroup3LargeClass5Alpha.GetValue() * 
            java.lang.Math.pow(m_iMaxDecayTime.GetValue(),
                m_fSpGroup3LargeClass5Beta.GetValue()) < -30.0) {
          throw new ModelException(ErrorGUI.BAD_DATA, "JAVA",
            "Alpha * (beta ^ max timesteps) < -30 must be true for the " +
            "following parameters: " + 
            m_fSpGroup3LargeClass5Alpha.GetDescriptor() + ", " + 
            m_fSpGroup3LargeClass5Beta.GetDescriptor());
        }
      }
      
      //Check alphas to make sure they are negative
      MakeSureLessThan(m_fScarSoilA, 0);
      MakeSureLessThan(m_fTipUpA, 0);
      if (bAnyGroup1) {
        MakeSureLessThan(m_fSpGroup1SmallClass1Alpha, 0);
        MakeSureLessThan(m_fSpGroup1SmallClass2Alpha, 0);
        MakeSureLessThan(m_fSpGroup1SmallClass3Alpha, 0);
        MakeSureLessThan(m_fSpGroup1SmallClass4Alpha, 0);
        MakeSureLessThan(m_fSpGroup1SmallClass5Alpha, 0);
        MakeSureLessThan(m_fSpGroup1LargeClass1Alpha, 0);
        MakeSureLessThan(m_fSpGroup1LargeClass2Alpha, 0);
        MakeSureLessThan(m_fSpGroup1LargeClass3Alpha, 0);
        MakeSureLessThan(m_fSpGroup1LargeClass4Alpha, 0);
        MakeSureLessThan(m_fSpGroup1LargeClass5Alpha, 0);
      }
      if (bAnyGroup2) {
        MakeSureLessThan(m_fSpGroup2SmallClass1Alpha, 0);
        MakeSureLessThan(m_fSpGroup2SmallClass2Alpha, 0);
        MakeSureLessThan(m_fSpGroup2SmallClass3Alpha, 0);
        MakeSureLessThan(m_fSpGroup2SmallClass4Alpha, 0);
        MakeSureLessThan(m_fSpGroup2SmallClass5Alpha, 0);
        MakeSureLessThan(m_fSpGroup2LargeClass1Alpha, 0);
        MakeSureLessThan(m_fSpGroup2LargeClass2Alpha, 0);
        MakeSureLessThan(m_fSpGroup2LargeClass3Alpha, 0);
        MakeSureLessThan(m_fSpGroup2LargeClass4Alpha, 0);
        MakeSureLessThan(m_fSpGroup2LargeClass5Alpha, 0);
      }
      if (bAnyGroup3) {
        MakeSureLessThan(m_fSpGroup3SmallClass1Alpha, 0);
        MakeSureLessThan(m_fSpGroup3SmallClass2Alpha, 0);
        MakeSureLessThan(m_fSpGroup3SmallClass3Alpha, 0);
        MakeSureLessThan(m_fSpGroup3SmallClass4Alpha, 0);
        MakeSureLessThan(m_fSpGroup3SmallClass5Alpha, 0);
        MakeSureLessThan(m_fSpGroup3LargeClass1Alpha, 0);
        MakeSureLessThan(m_fSpGroup3LargeClass2Alpha, 0);
        MakeSureLessThan(m_fSpGroup3LargeClass3Alpha, 0);
        MakeSureLessThan(m_fSpGroup3LargeClass4Alpha, 0);
        MakeSureLessThan(m_fSpGroup3LargeClass5Alpha, 0);
      }
      
      //Make sure initial conditions proportions add up to less than 1
      if (m_fSpGroup1SmallClass1InitLog.GetValue() +
          m_fSpGroup1SmallClass2InitLog.GetValue() +
          m_fSpGroup1SmallClass3InitLog.GetValue() +
          m_fSpGroup1SmallClass4InitLog.GetValue() +
          m_fSpGroup1SmallClass5InitLog.GetValue() +
          m_fSpGroup2SmallClass1InitLog.GetValue() +
          m_fSpGroup2SmallClass2InitLog.GetValue() +
          m_fSpGroup2SmallClass3InitLog.GetValue() +
          m_fSpGroup2SmallClass4InitLog.GetValue() +
          m_fSpGroup2SmallClass5InitLog.GetValue() +
          m_fSpGroup3SmallClass1InitLog.GetValue() +
          m_fSpGroup3SmallClass2InitLog.GetValue() +
          m_fSpGroup3SmallClass3InitLog.GetValue() +
          m_fSpGroup3SmallClass4InitLog.GetValue() +
          m_fSpGroup3SmallClass5InitLog.GetValue() +
          m_fSpGroup1LargeClass1InitLog.GetValue() +
          m_fSpGroup1LargeClass2InitLog.GetValue() +
          m_fSpGroup1LargeClass3InitLog.GetValue() +
          m_fSpGroup1LargeClass4InitLog.GetValue() +
          m_fSpGroup1LargeClass5InitLog.GetValue() +
          m_fSpGroup2LargeClass1InitLog.GetValue() +
          m_fSpGroup2LargeClass2InitLog.GetValue() +
          m_fSpGroup2LargeClass3InitLog.GetValue() +
          m_fSpGroup2LargeClass4InitLog.GetValue() +
          m_fSpGroup2LargeClass5InitLog.GetValue() +
          m_fSpGroup3LargeClass1InitLog.GetValue() +
          m_fSpGroup3LargeClass2InitLog.GetValue() +
          m_fSpGroup3LargeClass3InitLog.GetValue() +
          m_fSpGroup3LargeClass4InitLog.GetValue() +
          m_fSpGroup3LargeClass5InitLog.GetValue() +
          m_fInitCondScarSoil.GetValue() +
          m_fInitCondTipup.GetValue() > 1) {
        throw new ModelException(ErrorGUI.BAD_DATA, "JAVA",
            "The initial conditions values must not add up to be greater than 1.");
      }
      
      if (m_fSpGroup1SmallClass1PartCutLog.GetValue() +
          m_fSpGroup1SmallClass2PartCutLog.GetValue() +
          m_fSpGroup1SmallClass3PartCutLog.GetValue() +
          m_fSpGroup1SmallClass4PartCutLog.GetValue() +
          m_fSpGroup1SmallClass5PartCutLog.GetValue() +
          m_fSpGroup2SmallClass1PartCutLog.GetValue() +
          m_fSpGroup2SmallClass2PartCutLog.GetValue() +
          m_fSpGroup2SmallClass3PartCutLog.GetValue() +
          m_fSpGroup2SmallClass4PartCutLog.GetValue() +
          m_fSpGroup2SmallClass5PartCutLog.GetValue() +
          m_fSpGroup3SmallClass1PartCutLog.GetValue() +
          m_fSpGroup3SmallClass2PartCutLog.GetValue() +
          m_fSpGroup3SmallClass3PartCutLog.GetValue() +
          m_fSpGroup3SmallClass4PartCutLog.GetValue() +
          m_fSpGroup3SmallClass5PartCutLog.GetValue() +
          m_fSpGroup1LargeClass1PartCutLog.GetValue() +
          m_fSpGroup1LargeClass2PartCutLog.GetValue() +
          m_fSpGroup1LargeClass3PartCutLog.GetValue() +
          m_fSpGroup1LargeClass4PartCutLog.GetValue() +
          m_fSpGroup1LargeClass5PartCutLog.GetValue() +
          m_fSpGroup2LargeClass1PartCutLog.GetValue() +
          m_fSpGroup2LargeClass2PartCutLog.GetValue() +
          m_fSpGroup2LargeClass3PartCutLog.GetValue() +
          m_fSpGroup2LargeClass4PartCutLog.GetValue() +
          m_fSpGroup2LargeClass5PartCutLog.GetValue() +
          m_fSpGroup3LargeClass1PartCutLog.GetValue() +
          m_fSpGroup3LargeClass2PartCutLog.GetValue() +
          m_fSpGroup3LargeClass3PartCutLog.GetValue() +
          m_fSpGroup3LargeClass4PartCutLog.GetValue() +
          m_fSpGroup3LargeClass5PartCutLog.GetValue() +
          m_fPartialCutScarSoil.GetValue() +
          m_fPartialCutTipup.GetValue() > 1) {
        throw new ModelException(ErrorGUI.BAD_DATA, "JAVA",
            "The partial cut values must not add up to be greater than 1.");
      }

      if (m_fSpGroup1SmallClass1GapCutLog.GetValue() +
          m_fSpGroup1SmallClass2GapCutLog.GetValue() +
          m_fSpGroup1SmallClass3GapCutLog.GetValue() +
          m_fSpGroup1SmallClass4GapCutLog.GetValue() +
          m_fSpGroup1SmallClass5GapCutLog.GetValue() +
          m_fSpGroup2SmallClass1GapCutLog.GetValue() +
          m_fSpGroup2SmallClass2GapCutLog.GetValue() +
          m_fSpGroup2SmallClass3GapCutLog.GetValue() +
          m_fSpGroup2SmallClass4GapCutLog.GetValue() +
          m_fSpGroup2SmallClass5GapCutLog.GetValue() +
          m_fSpGroup3SmallClass1GapCutLog.GetValue() +
          m_fSpGroup3SmallClass2GapCutLog.GetValue() +
          m_fSpGroup3SmallClass3GapCutLog.GetValue() +
          m_fSpGroup3SmallClass4GapCutLog.GetValue() +
          m_fSpGroup3SmallClass5GapCutLog.GetValue() +
          m_fSpGroup1LargeClass1GapCutLog.GetValue() +
          m_fSpGroup1LargeClass2GapCutLog.GetValue() +
          m_fSpGroup1LargeClass3GapCutLog.GetValue() +
          m_fSpGroup1LargeClass4GapCutLog.GetValue() +
          m_fSpGroup1LargeClass5GapCutLog.GetValue() +
          m_fSpGroup2LargeClass1GapCutLog.GetValue() +
          m_fSpGroup2LargeClass2GapCutLog.GetValue() +
          m_fSpGroup2LargeClass3GapCutLog.GetValue() +
          m_fSpGroup2LargeClass4GapCutLog.GetValue() +
          m_fSpGroup2LargeClass5GapCutLog.GetValue() +
          m_fSpGroup3LargeClass1GapCutLog.GetValue() +
          m_fSpGroup3LargeClass2GapCutLog.GetValue() +
          m_fSpGroup3LargeClass3GapCutLog.GetValue() +
          m_fSpGroup3LargeClass4GapCutLog.GetValue() +
          m_fSpGroup3LargeClass5GapCutLog.GetValue() +
          m_fGapCutScarSoil.GetValue() +
          m_fGapCutTipup.GetValue() > 1) {
        throw new ModelException(ErrorGUI.BAD_DATA, "JAVA",
            "The gap cut values must not add up to be greater than 1.");
      }

      if (m_fSpGroup1SmallClass1ClearCutLog.GetValue() +
          m_fSpGroup1SmallClass2ClearCutLog.GetValue() +
          m_fSpGroup1SmallClass3ClearCutLog.GetValue() +
          m_fSpGroup1SmallClass4ClearCutLog.GetValue() +
          m_fSpGroup1SmallClass5ClearCutLog.GetValue() +
          m_fSpGroup2SmallClass1ClearCutLog.GetValue() +
          m_fSpGroup2SmallClass2ClearCutLog.GetValue() +
          m_fSpGroup2SmallClass3ClearCutLog.GetValue() +
          m_fSpGroup2SmallClass4ClearCutLog.GetValue() +
          m_fSpGroup2SmallClass5ClearCutLog.GetValue() +
          m_fSpGroup3SmallClass1ClearCutLog.GetValue() +
          m_fSpGroup3SmallClass2ClearCutLog.GetValue() +
          m_fSpGroup3SmallClass3ClearCutLog.GetValue() +
          m_fSpGroup3SmallClass4ClearCutLog.GetValue() +
          m_fSpGroup3SmallClass5ClearCutLog.GetValue() +
          m_fSpGroup1LargeClass1ClearCutLog.GetValue() +
          m_fSpGroup1LargeClass2ClearCutLog.GetValue() +
          m_fSpGroup1LargeClass3ClearCutLog.GetValue() +
          m_fSpGroup1LargeClass4ClearCutLog.GetValue() +
          m_fSpGroup1LargeClass5ClearCutLog.GetValue() +
          m_fSpGroup2LargeClass1ClearCutLog.GetValue() +
          m_fSpGroup2LargeClass2ClearCutLog.GetValue() +
          m_fSpGroup2LargeClass3ClearCutLog.GetValue() +
          m_fSpGroup2LargeClass4ClearCutLog.GetValue() +
          m_fSpGroup2LargeClass5ClearCutLog.GetValue() +
          m_fSpGroup3LargeClass1ClearCutLog.GetValue() +
          m_fSpGroup3LargeClass2ClearCutLog.GetValue() +
          m_fSpGroup3LargeClass3ClearCutLog.GetValue() +
          m_fSpGroup3LargeClass4ClearCutLog.GetValue() +
          m_fSpGroup3LargeClass5ClearCutLog.GetValue() +
          m_fClearCutScarSoil.GetValue() +
          m_fClearCutTipup.GetValue() > 1) {
        throw new ModelException(ErrorGUI.BAD_DATA, "JAVA",
            "The clear cut values must not add up to be greater than 1.");
      }
      
      //Make sure that the mean log diameters for small and large logs straddle
      //the size class boundary
      MakeSureIsBounded(m_fInitSmallLogMeanDiam, 0, m_fLogSizeClassBoundary.GetValue());
      MakeSureGreaterThan(m_fInitLargeLogMeanDiam, m_fLogSizeClassBoundary.GetValue());
      
      //Make sure the tree decay class values sum to 1
      if (java.lang.Math.abs(
      		(m_fPropFallenTreesDecayClass1.GetValue() +
      		m_fPropFallenTreesDecayClass2.GetValue() +
          m_fPropFallenTreesDecayClass3.GetValue() +
          m_fPropFallenTreesDecayClass4.GetValue() +
          m_fPropFallenTreesDecayClass5.GetValue()) - 1) > 0.00001) {
      	throw new ModelException(ErrorGUI.BAD_DATA, "JAVA",
            "The tree decay class values must add up to 1.");
      }
      
      //Make sure the snag decay class values sum to 1
      if (java.lang.Math.abs(
      		(m_fPropFallenSnagsDecayClass1.GetValue() +
      		m_fPropFallenSnagsDecayClass2.GetValue() +
          m_fPropFallenSnagsDecayClass3.GetValue() +
          m_fPropFallenSnagsDecayClass4.GetValue() +
          m_fPropFallenSnagsDecayClass5.GetValue()) - 1) > 0.00001) {
      	throw new ModelException(ErrorGUI.BAD_DATA, "JAVA",
            "The snag decay class values must add up to 1.");
      }
      
      //More to do if Harvest is enabled
      if (m_oManager.GetDisturbanceBehaviors().GetBehaviorByKey("Harvest").m_bIsEnabled) {
        //Make sure that the mean log diameters for small and large logs 
      	//straddle the size class boundary
        MakeSureIsBounded(m_fPartCutSmallLogMeanDiam, 0, m_fLogSizeClassBoundary.GetValue());
        MakeSureIsBounded(m_fGapCutSmallLogMeanDiam, 0, m_fLogSizeClassBoundary.GetValue());
        MakeSureIsBounded(m_fClearCutSmallLogMeanDiam, 0, m_fLogSizeClassBoundary.GetValue());
        MakeSureGreaterThan(m_fPartCutLargeLogMeanDiam, m_fLogSizeClassBoundary.GetValue());
        MakeSureGreaterThan(m_fGapCutLargeLogMeanDiam, m_fLogSizeClassBoundary.GetValue());
        MakeSureGreaterThan(m_fClearCutLargeLogMeanDiam, m_fLogSizeClassBoundary.GetValue());
      }
      
      //Make sure the snag dynamics behavior is enabled
      if (m_oManager.GetSnagDynamicsBehaviors()
      		.GetBehaviorByKey("Snag Decay Class Dynamics").m_bIsEnabled == false) {
      	throw new ModelException(ErrorGUI.BAD_DATA, "JAVA",
        "The behavior \"" + m_oManager.GetSnagDynamicsBehaviors()
    		.GetBehaviorByKey("Snag Decay Class Dynamics").GetDescriptor() + 
    		"\" must be used with the \"" + 
    		GetBehaviorByKey("Detailed Substrate").GetDescriptor() + "\".");
      }
    }
    
    //Check to see if the substrate calculations grids need updating - it
    //depends on what the user entered for max decay time
    Grid oGrid = m_oManager.GetGridByName("detailedsubstratecalcs");
    DataMember[] p_oDataMembers = oGrid.GetDataMembers();
    int iDecayTimesteps = ((int) java.lang.Math.ceil( (float) m_iMaxDecayTime.
        GetValue() / m_oManager.GetPlot().GetNumberOfYearsPerTimestep())) + 1;
		if (p_oDataMembers.length != 31 + (30*iDecayTimesteps)) {
			int i, j, k, iCount;
			p_oDataMembers = new DataMember[31 + (30*iDecayTimesteps)];

	    p_oDataMembers[0] = new DataMember("Amount of new tip-up mounds",
	                                       "newtipup",
	                                       DataMember.FLOAT);

	    iCount = 1;
	    for (i = 1; i <= 5; i++) { //decay class
	    	for (j = 1; j <= 3; j++) { //species group
	    		p_oDataMembers[iCount] = new DataMember(
	    				"New Log Area Sp Group " + j + " Small Decay " + i, 
	    				"newloggroup" + j + "smalldecay" + i, DataMember.FLOAT);     		
	    		iCount++;
	    		p_oDataMembers[iCount] = new DataMember(
	    				"New Log Area Sp Group " + j + " Large Decay " + i, 
	    				"newloggroup" + j + "largedecay" + i, DataMember.FLOAT);     		
	    		iCount++;
	    		for (k = 0; k < iDecayTimesteps; k++) {
	    		  p_oDataMembers[iCount] = new DataMember(
	    		  	"Prop Sp Group " + j + " Small Decay " + i + " Timestep " + k, 
	    				"loggroup" + j + "smalldecay" + i + "_" + k, DataMember.FLOAT);     		
	    		  iCount++;
	    		  p_oDataMembers[iCount] = new DataMember(
	      		  	"Prop Sp Group " + j + " Large Decay " + i + " Timestep " + k, 
	      				"loggroup" + j + "largedecay" + i + "_" + k, DataMember.FLOAT);     		
	      		  iCount++;
	    		}
	    	}
	    }
	    oGrid.SetDataMembers(p_oDataMembers);
      //Go to the output behavior - if detailedsubstratecalcs is a grid, erase 
	    //any data members that may have been removed
	    OutputBehaviors oOutput = m_oManager.GetOutputBehaviors();
			DetailedGridSettings oSettings;
			DataMember oMember;
			boolean bFound;
			for (i = 0; i < oOutput.mp_oDetailedGridSaveSettings.size(); i++) {
				oSettings = (DetailedGridSettings) oOutput.mp_oDetailedGridSaveSettings.elementAt(i);
				if (oSettings.GetName().equals("detailedsubstratecalcs")) {
					for (j = 0; j < oSettings.GetNumberOfFloats(); j++) {
						oMember = (DataMember) oSettings.GetFloat(j);
						bFound = false;
						for (k = 0; k < p_oDataMembers.length; k++) {
							if (p_oDataMembers[k].equals(oMember)) {
								bFound = true; break;
							}
						}
						if (bFound == false) {
							oSettings.RemoveFloat(j);
							j--;
						}
					}
				}
			}
		}
    
  }
}
